@@ -1,4 +1,4 @@
-# $Id: nexus.pm,v 1.12 2002/10/22 07:38:25 lapp Exp $
+# $Id$
 #
 # BioPerl module for Bio::AlignIO::nexus
 #
@@ -93,8 +93,11 @@ sub next_aln {
 
     my $aln =  Bio::SimpleAlign->new(-source => 'nexus');
 
-    # file starts with #NEXUS
+    # file starts with '#NEXUS' but we allow white space only lines before it
     $entry = $self->_readline;
+    $entry = $self->_readline while $entry =~ /^\s+$/;
+
+    return unless $entry;
     $self->throw("Not a valid interleaved NEXUS file! [#NEXUS] not starting the file\n$entry")
 	unless $entry =~ /^#NEXUS/i;
 
@@ -237,6 +240,10 @@ seqcount [$count] > predeclared [$seqcount] ") if $count > $seqcount;
 	$aln->map_chars($1, $2) while $equate =~ /(\S)=(\S)/g;
     }
 
+    while  ($entry !~ /endblock/i) {
+        $entry = $self->_readline;
+    }
+
     return $aln;
 }
 
