@@ -1,4 +1,5 @@
-# $Id: AnnotationCollectionI.pm,v 1.7.2.1 2002/06/10 17:18:56 jason Exp $
+# $Id$
+
 #
 # BioPerl module for Bio::AnnotationCollectionI
 #
@@ -100,7 +101,7 @@ the bugs and their resolution.  Bug reports can be submitted via email
 or the web:
 
   bioperl-bugs@bio.perl.org
-  http://bio.perl.org/bioperl-bugs/
+  http://bugzilla.bioperl.org/
 
 =head1 AUTHOR - Ewan Birney
 
@@ -140,8 +141,7 @@ use Bio::Root::RootI;
 =cut
 
 sub get_all_annotation_keys{
-    my ($self) = @_;
-    $self->throw("Abstract method get_all_annotation_keys implementing class did not provide method");
+    shift->throw_not_implemented();
 }
 
 
@@ -156,10 +156,7 @@ sub get_all_annotation_keys{
 =cut
 
 sub get_Annotations{
-    my ($self) = @_;
-
-    $self->throw("Abstract method get_Annotations implementing class did not provide method");
-    
+    shift->throw_not_implemented();    
 }
 
 =head2 get_num_of_annotations
@@ -174,8 +171,7 @@ sub get_Annotations{
 =cut
 
 sub get_num_of_annotations{
-   my ($self) = @_;
-       $self->throw("Abstract method get_num_of_annotations implementing class did not provide method");
+    shift->throw_not_implemented();
 }
 
 1;
