@@ -1,4 +1,4 @@
-# $Id: OBDAIndex.pm,v 1.4.2.3 2002/03/15 14:40:02 heikki Exp $
+# $Id$
 #
 # BioPerl module for Bio::DB::Flat::OBDAIndex
 #
@@ -165,7 +165,7 @@ the bugs and their resolution.  Bug reports can be submitted via
 email or the web:
 
   bioperl-bugs@bio.perl.org
-  http://bio.perl.org/bioperl-bugs/
+  http://bugzilla.bioperl.org/
 
 =head1 AUTHOR - Michele Clamp
 
@@ -183,7 +183,11 @@ package Bio::DB::Flat::OBDAIndex;
 use strict;
 use vars qw(@ISA);
 
-use FileHandle;
+use Fcntl qw(SEEK_END SEEK_CUR);
+# rather than using tell which might be buffered
+sub systell{ sysseek($_[0], 0, SEEK_CUR) }
+sub syseof{ sysseek($_[0], 0, SEEK_END) }
+
 
 use Bio::DB::RandomAccessI;
 use Bio::Root::RootI;
@@ -195,7 +199,7 @@ use Bio::Seq;
 use constant CONFIG_FILE_NAME => 'config.dat';
 use constant HEADER_SIZE      => 4;
 
-my @formats = {'FASTA','SWISSPROT','EMBL'};
+my @formats = ['FASTA','SWISSPROT','EMBL'];
 
 =head2 new
 
@@ -240,15 +244,16 @@ sub new {
 
     bless $self, $class;
 
-    my ($index_dir,$dbname,$format,$primary_pattern,$primary_namespace,$start_pattern,$secondary_patterns) =  
-	$self->_rearrange([qw(INDEX_DIR
-			      DBNAME
-			      FORMAT
-			      PRIMARY_PATTERN
-			      PRIMARY_NAMESPACE
-			      START_PATTERN
-			      SECONDARY_PATTERNS)], @args);
-
+    my ($index_dir,$dbname,$format,$primary_pattern,$primary_namespace,
+	$start_pattern,$secondary_patterns) =  
+	    $self->_rearrange([qw(INDEX_DIR
+				  DBNAME
+				  FORMAT
+				  PRIMARY_PATTERN
+				  PRIMARY_NAMESPACE
+				  START_PATTERN
+				  SECONDARY_PATTERNS)], @args);
+    
     $self->index_directory($index_dir);
     $self->database_name     ($dbname);
 
@@ -296,17 +301,22 @@ sub new_from_registry {
 
 sub get_Seq_by_id {
     my ($self,$id) = @_;
-
+   
     my ($fh,$length) = $self->get_stream_by_id($id);
-
+    
     if (!defined($self->format)) {
 	$self->throw("Can't create sequence - format is not defined");
     }
-
+   
+    if(!$fh){
+      return;
+    }
     if (!defined($self->{_seqio})) {
+     
 	$self->{_seqio} = new Bio::SeqIO(-fh => $fh,
 					 -format => $self->format);
     } else {
+      
 	$self->{_seqio}->fh($fh);
     }
     
@@ -356,9 +366,9 @@ sub get_stream_by_id {
 
     my $indexfh = $self->primary_index_filehandle;
 
-    sysseek ($indexfh,0,2);
+    syseof ($indexfh);
 
-    my $filesize = (tell $indexfh);
+    my $filesize = systell($indexfh);
 
     my $end = ($filesize-$self->{_start_pos})/$self->record_size;
 
@@ -367,9 +377,9 @@ sub get_stream_by_id {
     
     my ($fileid,$pos,$length) = split(/\t/,$rest);
 
-#    print "Found id entry $newid $fileid $pos $length:$rest\n";
+    #print STDERR "OBDAIndex Found id entry $newid $fileid $pos $length:$rest\n";
 
-    if (!defined($newid)) {
+    if (!$newid) {
       return;
     }
 
@@ -438,9 +448,9 @@ sub get_Seq_by_secondary {
 
     my $fh = $self->open_secondary_index($name);
 
-    sysseek ($fh,0,2);
+    syseof ($fh);
 
-    my $filesize = (tell $fh);
+    my $filesize = systell($fh);
 
     my $recsize = $self->{_secondary_record_size}{$name};
 #    print "Name " . $recsize . "\n";
@@ -635,7 +645,7 @@ sub make_index {
     if (! -d $rootdir) {
 	$self->throw("Index directory [$rootdir] is not a directory. Cant' build indices");
     }
-    if (!defined(@files)) {
+    if (!(@files)) {
 	$self->throw("Must enter an array of filenames to index");
     }
     
@@ -699,7 +709,7 @@ sub _index_file {
     my $new_primary_entry;
     
     my $length;
-    my $pos = 0;
+    #my $pos = 0;
     my $fh = \*FILE;
 
     my $done = -1;
@@ -1116,7 +1126,7 @@ sub make_config_file {
 
     my @second = keys %$second_patterns;
 
-    if (defined(@second))  {
+    if ((@second))  {
 	print CON "secondary_namespaces";
 
 	foreach my $second (@second) {
@@ -1232,7 +1242,7 @@ sub read_config_file {
 
     my @fileid_keys = keys (%{$self->{_fileid}});
     
-    if (!defined(@fileid_keys)) {
+    if (!(@fileid_keys)) {
 	$self->throw("No flatfile fileid files in config - check the index has been made correctly");
     }
 
@@ -1286,7 +1296,7 @@ sub get_filehandle_by_fileid {
     if (!defined($self->{_fileid}{$fileid})) {
 	$self->throw("ERROR: undefined fileid in index [$fileid]");
     }
-
+   
     return $self->{_fileid}{$fileid};
 }
 
