@@ -25,11 +25,11 @@ use strict;
 use Bio::Root::Root;
 use Bio::Annotation::Collection;
 use Bio::RangeI;
+use Bio::Das::SegmentI;
 use Bio::SeqI;
 
-use vars qw($VERSION @ISA);
-@ISA = qw(Bio::Root::Root Bio::RangeI Bio::SeqI);
-$VERSION = '0.30';
+use vars qw(@ISA);
+@ISA = qw(Bio::Root::Root Bio::RangeI Bio::SeqI Bio::Das::SegmentI);
 
 use overload 
   '""'     => 'asString',
@@ -102,6 +102,7 @@ to create the segment.
 sub factory { shift->{factory} }
 
 # start, stop, length
+
 =head2 start
 
  Title   : start
@@ -178,7 +179,7 @@ Returns the strand on which the segment resides, either +1, 0 or -1.
 
 sub strand {
   my $self = shift;
-  return $self->stop <=> $self->start;
+  0;
 }
 
 =head2 low
@@ -239,17 +240,22 @@ sub sourceseq { shift->{sourceseq} }
 =head2 class
 
  Title   : class
- Usage   : $s->class
+ Usage   : $s->class([$newclass])
  Function: get the source sequence class
  Returns : a string
- Args    : none
+ Args    : new class (optional)
  Status  : Public
 
-Returns the class for the source sequence for this segment.
+Gets or sets the class for the source sequence for this segment.
 
 =cut
 
-sub class     { shift->{class}     }
+sub class     { 
+  my $self = shift;
+  my $d = $self->{class};
+  $self->{class} = shift if @_;
+  $d;
+}
 
 =head2 subseq
 
@@ -313,6 +319,39 @@ sub seq {
 
 *protein = *dna = \&seq;
 
+
+=head2 primary_seq
+
+ Title   : primary_seq
+ Usage   : $s->primary_seq
+ Function: returns a Bio::PrimarySeqI compatible object
+ Returns : a Bio::PrimarySeqI object
+ Args    : none
+ Status  : Public
+
+This is for compatibility with BioPerl's separation of SeqI
+from PrimarySeqI.  It just returns itself.
+
+=cut
+
+sub primary_seq { shift }
+
+=head2 type
+
+ Title   : type
+ Usage   : $s->type
+ Function: return the string "feature"
+ Returns : the string "feature"
+ Args    : none
+ Status  : Public
+
+This is for future sequence ontology-compatibility and
+represents the default type of a feature on the genome
+
+=cut
+
+sub type { "feature" }
+
 =head2 equals
 
  Title   : equals
@@ -423,10 +462,10 @@ Bio::DB::GFF::RelSegment.
 
 *abs_start  = \&start;
 
-=head2 abs_stop
+=head2 abs_end
 
- Title   : abs_stop
- Usage   : $s->abs_stop
+ Title   : abs_end
+ Usage   : $s->abs_end
  Function: the absolute stop of the segment
  Returns : an integer
  Args    : none
@@ -456,7 +495,7 @@ Bio::DB::GFF::RelSegment.
 
 sub abs_strand {
   my $self = shift;
-  return $self->abs_stop <=> $self->abs_start;
+  return $self->abs_end <=> $self->abs_start;
 }
 
 =head2 abs_ref
@@ -520,7 +559,7 @@ An alias for refseq() but only allows reading.
 
 =cut
 
-sub seq_id { shift->ref }
+sub seq_id { shift->refseq }
 
 =head2 truncated
 
@@ -642,10 +681,10 @@ sub primary_id {
 }
 
 
-=head2 display_id
+=head2 display_name
 
- Title   : display_id
- Usage   : $id = $obj->display_id or $obj->display_id($newid);
+ Title   : display_name
+ Usage   : $id = $obj->display_name or $obj->display_name($newid);
  Function: Gets or sets the display id, also known as the common name of
            the Seq object.
 
@@ -665,10 +704,13 @@ sub primary_id {
  Returns : A string
  Args    : None or a new id
 
+Note, this used to be called display_id(), and this name is preserved for
+backward compatibility.  The default is to return the seq_id().
 
 =cut
 
-sub display_id { shift->seq_id }
+sub display_name { shift->seq_id }
+*display_id = \&display_name;
 
 =head2 accession_number
 
