@@ -1,8 +1,8 @@
-# $Id: DriverFactory.pm,v 1.8 2001/12/14 16:40:14 heikki Exp $
+# $Id$
 #
 # BioPerl module for Bio::Factory::DriverFactory
 #
-# Cared for by Jason Stajich <jason@chg.mc.duke.edu> and
+# Cared for by Jason Stajich <jason@bioperl.org> and
 #              Hilmar Lapp <hlapp@gmx.net>
 #
 # Copyright Jason Stajich, Hilmar Lapp
@@ -43,11 +43,11 @@ the bugs and their resolution.  Bug reports can be submitted via email
 or the web:
 
   bioperl-bugs@bio.perl.org
-  http://bio.perl.org/bioperl-bugs/
+  http://bugzilla.bioperl.org/
 
 =head1 AUTHOR - Jason Stajich
 
-Email Jason Stajich E<lt>jason@chg.mc.duke.eduE<gt>
+Email Jason Stajich E<lt>jason@bioperl.orgE<gt>
 
 =head1 APPENDIX
 
