@@ -1,6 +1,6 @@
 package Bio::Graphics::FeatureFile;
 
-# $Id: FeatureFile.pm,v 1.1.2.14 2002/07/04 14:40:00 lstein Exp $
+# $Id$
 # This package parses and renders a simple tab-delimited format for features.
 # It is simpler than GFF, but still has a lot of expressive power.
 # See __END__ for the file format
@@ -39,7 +39,9 @@ Bio::Graphics::FeatureFile -- A set of Bio::Graphics features, stored in a file
  $data->add_type(EST => {fgcolor=>'blue',height=>12});
 
  # add a feature
- my $feature = Bio::Graphics::Feature->new(....); # or some other SeqI
+ my $feature = Bio::Graphics::Feature->new(
+                                             # params
+                                          ); # or some other SeqI
  $data->add_feature($feature=>'EST');
 
 =head1 DESCRIPTION
@@ -118,8 +120,6 @@ use Bio::Graphics::Panel;
 use Carp;
 use IO::File;
 use Text::Shellwords;
-use vars '$VERSION';
-$VERSION = '1.02';
 
 # default colors for unconfigured features
 my @COLORS = qw(cyan blue red yellow green wheat turquoise orange);
@@ -129,10 +129,10 @@ use constant WIDTH => 600;
 
 =over 4
 
-=item $features = Bio::Graphics::FeatureFile->new(@args)
+=item $features = Bio::Graphics::FeatureFile-E<gt>new(@args)
 
 Create a new Bio::Graphics::FeatureFile using @args to initialize the
-object.  Arguments are -name=>value pairs:
+object.  Arguments are -name=E<gt>value pairs:
 
   Argument         Value
   --------         -----
@@ -151,7 +151,7 @@ object.  Arguments are -name=>value pairs:
 
    -safe           Indicates that the contents of this file is trusted.
                    Any option value that begins with the string "sub {"
-                   will be evaluated as a code reference.
+                   or \&subname will be evaluated as a code reference.
 
 The -file and -text arguments are mutually exclusive, and -file will
 supersede the other if both are present.
@@ -168,6 +168,11 @@ The -smart_features flag is used by the generic genome browser to
 provide features with a way to access the link-generation code.  See
 gbrowse for how this works.
 
+If the file is trusted, and there is an option named "init_code" in
+the [GENERAL] section of the file, it will be evaluated as perl code
+immediately after parsing.  You can use this to declare global
+variables and subroutines for use in option values.
+
 =back
 
 =cut
@@ -222,9 +227,10 @@ sub new {
 
 # render our features onto a panel using configuration data
 # return the number of tracks inserted
+
 =over 4
 
-=item ($rendered,$panel) = $features->render([$panel])
+=item ($rendered,$panel) = $features-E<gt>render([$panel])
 
 Render features in the data set onto the indicated
 Bio::Graphics::Panel.  If no panel is specified, creates one.
@@ -238,6 +244,8 @@ for you.
 
 =cut
 
+#"
+
 sub render {
   my $self = shift;
   my $panel = shift;
@@ -306,7 +314,7 @@ sub _stat {
 
 =over 4
 
-=item $error = $features->error([$error])
+=item $error = $features-E<gt>error([$error])
 
 Get/set the current error message.
 
@@ -323,7 +331,7 @@ sub error {
 
 =over 4
 
-=item $smart_features = $features->smart_features([$flag]
+=item $smart_features = $features-E<gt>smart_features([$flag]
 
 Get/set the "smart_features" flag.  If this is set, then any features
 added to the featurefile object will have their configurator() method
@@ -369,7 +377,7 @@ sub parse_text {
   my $text = shift;
 
   $self->init_parse;
-  foreach (split /\r?\n|\r\n?/,$text) {
+  foreach (split /\015?\012|\015\012?/,$text) {
     $self->parse_line($_);
   }
   $self->finish_parse;
@@ -379,26 +387,29 @@ sub parse_line {
   my $self = shift;
   local $_ = shift;
 
-  s/\r//g;  # get rid of carriage returns left over by MS-DOS/Windows systems
+  s/\015//g;  # get rid of carriage returns left over by MS-DOS/Windows systems
 
-  return if /^[\#]/;
+  return if /^\s*[\#]/;
 
   if (/^\s+(.+)/ && $self->{current_tag}) { # continuation line
-      my $value = $1;
-      my $cc = $self->{current_config} ||= 'general';       # in case no configuration named
-      $self->{config}{$cc}{$self->{current_tag}} .= ' ' . $value;
-      return;
+    my $value = $1;
+    my $cc = $self->{current_config} ||= 'general';       # in case no configuration named
+    $self->{config}{$cc}{$self->{current_tag}} .= ' ' . $value;
+    # respect newlines in code subs
+    $self->{config}{$cc}{$self->{current_tag}} .= "\n"
+      if $self->{config}{$cc}{$self->{current_tag}}=~ /^sub\s*\{/;
+    return;
   }
 
   if (/^\s*\[([^\]]+)\]/) {  # beginning of a configuration section
-     my $label = $1;
-     my $cc = $label =~ /^(general|default)$/i ? 'general' : $label;  # normalize
-     push @{$self->{types}},$cc unless $cc eq 'general';
-     $self->{current_config} = $cc;
-     return;
+    my $label = $1;
+    my $cc = $label =~ /^(general|default)$/i ? 'general' : $label;  # normalize
+    push @{$self->{types}},$cc unless $cc eq 'general';
+    $self->{current_config} = $cc;
+    return;
   }
 
-  if (/^([\w ]+?)\s*=\s*(.*)/) {   # key value pair within a configuration section
+  if (/^([\w: -]+?)\s*=\s*(.*)/) {   # key value pair within a configuration section
     my $tag = lc $1;
     my $cc = $self->{current_config} ||= 'general';       # in case no configuration named
     my $value = defined $2 ? $2 : '';
@@ -480,17 +491,42 @@ sub parse_line {
   $type = '' unless defined $type;
   $name = '' unless defined $name;
 
+  # attribute handling
+  my %attributes;
+  my $score;
+  if (defined $description && $description =~ /\w+=\w+/) { # attribute line
+    my @attributes = split /;\s*/,$description;
+    foreach (@attributes) {
+      my ($name,$value) = split /=/,$_,2;
+      Bio::Root::Root->throw(qq("$_" is not a valid attribute=value pair)) unless defined $value;
+      _unescape($name);
+      my @values = split /,/,$value;
+      _unescape(@values);
+      if ($name =~ /^(note|description)/) {
+	$description = "@values";
+      } elsif ($name eq 'url') {
+	$url = $value;
+      } elsif ($name eq 'score') {
+	$score = $value;
+      } else {
+	push @{$attributes{$name}},@values;
+      }
+    }
+  }
+
   # either create a new feature or add a segment to it
   if (my $feature = $self->{seenit}{$type,$name}) {
     $feature->add_segment(@parts);
   } else {
     $feature = $self->{seenit}{$type,$name} = 
-      Bio::Graphics::Feature->new(-name     => $name,
-				  -type     => $type,
+      Bio::Graphics::Feature->new(-name       => $name,
+				  -type       => $type,
 				  $strand ? (-strand   => make_strand($strand)) : (),
-				  -segments => \@parts,
-				  -source   => $description,
-				  -ref      => $ref,
+				  defined $score ? (-score=>$score) : (),
+				  -segments   => \@parts,
+				  -desc       => $description,
+				  -ref        => $ref,
+				  -attributes => \%attributes,
 				  defined($url) ? (-url      => $url) : (),
 				 );
     $feature->configurator($self) if $self->smart_features;
@@ -502,10 +538,17 @@ sub parse_line {
   }
 }
 
+sub _unescape {
+  foreach (@_) {
+    tr/+/ /;       # pluses become spaces
+    s/%([0-9a-fA-F]{2})/chr hex($1)/g;
+  }
+  @_;
+}
 
 =over 4
 
-=item $features->add_feature($feature [=>$type])
+=item $features-E<gt>add_feature($feature [=E<gt>$type])
 
 Add a new Bio::FeatureI object to the set.  If $type is specified, the
 object will be added with the indicated type.  Otherwise, the
@@ -527,10 +570,10 @@ sub add_feature {
 
 =over 4
 
-=item $features->add_type($type=>$hashref)
+=item $features-E<gt>add_type($type=E<gt>$hashref)
 
 Add a new feature type to the set.  The type is a string, such as
-"EST".  The hashref is a set of key=>value pairs indicating options to
+"EST".  The hashref is a set of key=E<gt>value pairs indicating options to
 set on the type.  Example:
 
   $features->add_type(EST => { glyph => 'generic', fgcolor => 'blue'})
@@ -561,7 +604,7 @@ sub add_type {
 
 =over 4
 
-=item $features->set($type,$tag,$value)
+=item $features-E<gt>set($type,$tag,$value)
 
 Change an individual option for a particular type.  For example, this
 will change the foreground color of EST features to my favorite color:
@@ -596,7 +639,7 @@ sub DESTROY { shift->destroy(@_) }
 
 =over 4
 
-=item $value = $features->setting($stanza => $option)
+=item $value = $features-E<gt>setting($stanza =E<gt> $option)
 
 In the two-element form, the setting() method returns the value of an
 option in the configuration stanza indicated by $stanza.  For example:
@@ -640,7 +683,7 @@ sub _setting {
 
 =over 4
 
-=item $value = $features->code_setting($stanza=>$option);
+=item $value = $features-E<gt>code_setting($stanza=E<gt>$option);
 
 This works like setting() except that it is also able to evaluate code
 references.  These are options whose values begin with the characters
@@ -659,16 +702,28 @@ sub code_setting {
   my $setting = $self->_setting($section=>$option);
   return unless defined $setting;
   return $setting if ref($setting) eq 'CODE';
-  return $setting unless $setting =~ /^sub\s+\{/;
-  my $coderef = eval $setting;
-  warn $@ if $@;
-  $self->set($section,$option,$coderef);
-  return $coderef;
+  if ($setting =~ /^\\&(\w+)/) {  # coderef in string form
+    my $subroutine_name = $1;
+    my $package         = $self->base2package;
+    my $codestring      = "\\&${package}\:\:${subroutine_name}";
+    my $coderef         = eval $codestring;
+    warn $@ if $@;
+    $self->set($section,$option,$coderef);
+    return $coderef;
+  }
+  elsif ($setting =~ /^sub\s*\{/) {
+    my $coderef   = eval $setting;
+    warn $@ if $@;
+    $self->set($section,$option,$coderef);
+    return $coderef;
+  } else {
+    return $setting;
+  }
 }
 
 =over 4
 
-=item $flag = $features->safe([$flag]);
+=item $flag = $features-E<gt>safe([$flag]);
 
 This gets or sets and "safe" flag.  If the safe flag is set, then
 calls to setting() will invoke code_setting(), allowing values that
@@ -691,11 +746,11 @@ sub safe {
 
 =over 4
 
-=item @args = $features->style($type)
+=item @args = $features-E<gt>style($type)
 
 Given a feature type, returns a list of track configuration arguments
 suitable for suitable for passing to the
-Bio::Graphics::Panel->add_track() method.
+Bio::Graphics::Panel-E<gt>add_track() method.
 
 =back
 
@@ -715,10 +770,10 @@ sub style {
 
 =over 4
 
-=item $glyph = $features->glyph($type);
+=item $glyph = $features-E<gt>glyph($type);
 
 Return the name of the glyph corresponding to the given type (same as
-$features->setting($type=>'glyph')).
+$features-E<gt>setting($type=E<gt>'glyph')).
 
 =back
 
@@ -736,7 +791,7 @@ sub glyph {
 
 =over 4
 
-=item @types = $features->configured_types()
+=item @types = $features-E<gt>configured_types()
 
 Return a list of all the feature types currently known to the feature
 file set.  Roughly equivalent to:
@@ -756,7 +811,7 @@ sub configured_types {
 
 =over 4
 
-=item  @types = $features->types()
+=item  @types = $features-E<gt>types()
 
 This is similar to the previous method, but will return *all* feature
 types, including those that are not configured with a stanza.
@@ -774,7 +829,7 @@ sub types {
 
 =over 4
 
-=item @features = $features->features($type)
+=item @features = $features-E<gt>features($type)
 
 Return a list of all the feature types of type "$type".  If the
 featurefile object was created by parsing a file or text scalar, then
@@ -804,7 +859,7 @@ sub make_strand {
 
 =over 4
 
-=item @refs = $features->refs
+=item @refs = $features-E<gt>refs
 
 Return the list of reference sequences referred to by this data file.
 
@@ -820,7 +875,7 @@ sub refs {
 
 =over 4
 
-=item  $min = $features->min
+=item  $min = $features-E<gt>min
 
 Return the minimum coordinate of the leftmost feature in the data set.
 
@@ -832,7 +887,7 @@ sub min { shift->{min} }
 
 =over 4
 
-=item $max = $features->max
+=item $max = $features-E<gt>max
 
 Return the maximum coordinate of the rightmost feature in the data set.
 
@@ -873,6 +928,7 @@ sub consolidate_groups {
 
 sub evaluate_coderefs {
   my $self = shift;
+  $self->initialize_code();
   for my $s ($self->_setting) {
     for my $o ($self->_setting($s)) {
       $self->code_setting($s,$o);
@@ -880,6 +936,22 @@ sub evaluate_coderefs {
   }
 }
 
+sub initialize_code {
+  my $self       = shift;
+  my $package = $self->base2package;
+  my $init_code = $self->_setting(general => 'init_code') or return;
+  my $code = "package $package; $init_code; 1;";
+  eval $code;
+  warn $@ if $@;
+}
+
+sub base2package {
+  my $self = shift;
+  (my $package = overload::StrVal($self)) =~ s/[^a-z0-9A-Z_]/_/g;
+  $package     =~ s/^[^a-zA-Z_]/_/g;
+  $package;
+}
+
 sub split_group {
   my $group = shift;
 
@@ -955,13 +1027,13 @@ sub new_panel {
 
 =over 4
 
-=item $mtime = $features->mtime
+=item $mtime = $features-E<gt>mtime
 
-=item $atime = $features->atime
+=item $atime = $features-E<gt>atime
 
-=item $ctime = $features->ctime
+=item $ctime = $features-E<gt>ctime
 
-=item $size = $features->size
+=item $size = $features-E<gt>size
 
 Returns stat() information about the data file, for featurefile
 objects created using the -file option.  Size is in bytes.  mtime,
@@ -983,7 +1055,7 @@ sub size  { shift->{stat}->[7];  }
 
 =over 4
 
-=item $label = $features->feature2label($feature)
+=item $label = $features-E<gt>feature2label($feature)
 
 Given a feature, determines the configuration stanza that bests
 describes it.  Uses the feature's type() method if it has it (DasI
@@ -998,13 +1070,15 @@ sub feature2label {
   my $feature = shift;
   my $type  = eval {$feature->type} || $feature->primary_tag or return;
   (my $basetype = $type) =~ s/:.+$//;
-  my $label = $self->type2label($type) || $self->type2label($basetype) || $type;
-  $label;
+  my @labels = $self->type2label($type);
+  @labels = $self->type2label($basetype) unless @labels;
+  @labels = ($type) unless @labels;;
+  wantarray ? @labels : $labels[0];
 }
 
 =over 4
 
-=item $link = $features->make_link($feature)
+=item $link = $features-E<gt>make_link($feature)
 
 Given a feature, tries to generate a URL to link out from it.  This
 uses the 'link' option, if one is present.  This method is a
@@ -1017,33 +1091,47 @@ convenience for the generic genome browser.
 sub make_link {
   my $self     = shift;
   my $feature  = shift;
-  my $label    = $self->feature2label($feature) or return;
-  my $link     = $self->setting($label,'link');
-  $link        = $self->setting(general=>'link') unless defined $link;
-  return unless $link;
-  return $self->link_pattern($link,$feature);
+  for my $label ($self->feature2label($feature)) {
+    my $link     = $self->setting($label,'link');
+    $link        = $self->setting(general=>'link') unless defined $link;
+    next unless $link;
+    return $self->link_pattern($link,$feature);
+  }
+  return;
 }
 
 sub link_pattern {
   my $self = shift;
-  my ($pattern,$feature) = @_;
+  my ($pattern,$feature,$panel) = @_;
+  require CGI unless defined &CGI::escape;
+  my $n;
   $pattern =~ s/\$(\w+)/
-    $1 eq 'name'   ? $feature->name
-      : $1 eq 'class'  ? $feature->class
-      : $1 eq 'type'   ? $feature->method
-      : $1 eq 'method' ? $feature->method
-      : $1 eq 'source' ? $feature->source
+    CGI::escape(
+    $1 eq 'ref'              ? ($n = $feature->location->seq_id) && "$n"
+      : $1 eq 'name'         ? ($n = $feature->display_name) && "$n"  # workaround broken CGI.pm
+      : $1 eq 'class'        ? eval {$feature->class}  || ''
+      : $1 eq 'type'         ? eval {$feature->method} || $feature->primary_tag
+      : $1 eq 'method'       ? eval {$feature->method} || $feature->primary_tag
+      : $1 eq 'source'       ? eval {$feature->source} || $feature->source_tag
+      : $1 eq 'start'        ? $feature->start
+      : $1 eq 'end'          ? $feature->end
+      : $1 eq 'stop'         ? $feature->end
+      : $1 eq 'segstart'     ? $panel->start
+      : $1 eq 'segend'       ? $panel->end
+      : $1 eq 'description'  ? eval {join '',$feature->notes} || ''
       : $1
+	       )
        /exg;
   return $pattern;
 }
 
-# given a feature type, return its label
+# given a feature type, return its label(s)
 sub type2label {
   my $self = shift;
   my $type = shift;
   $self->{_type2label} ||= $self->invert_types;
-  $self->{_type2label}{$type};
+  my @labels = keys %{$self->{_type2label}{$type}};
+  wantarray ? @labels : $labels[0]
 }
 
 sub invert_types {
@@ -1053,7 +1141,7 @@ sub invert_types {
   for my $label (keys %{$config}) {
     my $feature = $config->{$label}{feature} or next;
     foreach (shellwords($feature||'')) {
-      $inverted{$_} = $label;
+      $inverted{$_}{$label}++;
     }
   }
   \%inverted;
@@ -1061,7 +1149,7 @@ sub invert_types {
 
 =over 4
 
-=item $citation = $features->citation($feature)
+=item $citation = $features-E<gt>citation($feature)
 
 Given a feature, tries to generate a citation for it, using the
 "citation" option if one is present.  This method is a convenience for
@@ -1081,7 +1169,7 @@ sub citation {
 
 =over 4
 
-=item $name = $features->name([$feature])
+=item $name = $features-E<gt>name([$feature])
 
 Get/set the name of this feature set.  This is a convenience method
 useful for keeping track of multiple feature sets.
