@@ -1,5 +1,5 @@
 package Bio::Graphics::Glyph::transcript;
-# $Id: transcript.pm,v 1.3.2.11 2002/07/10 01:25:09 lstein Exp $
+# $Id$
 
 use strict;
 use Bio::Graphics::Glyph::segments;
@@ -10,7 +10,8 @@ sub pad_left  {
   my $self = shift;
   my $pad  = $self->SUPER::pad_left;
   return $pad if $self->{level} > 0;
-  return $pad unless $self->feature->strand < 0;
+  my $strand = $self->feature->strand;
+  return $pad unless defined $strand && $strand < 0;
   return $self->arrow_length > $pad ? $self->arrow_length : $pad;
 }
 
@@ -18,7 +19,8 @@ sub pad_right {
   my $self = shift;
   my $pad  = $self->SUPER::pad_right;
   return $pad if $self->{level} > 0;
-  return $pad unless $self->feature->strand > 0;
+  my $strand = $self->feature->strand;
+  return $pad unless defined($strand) && $strand > 0;
   return $self->arrow_length > $pad ? $self->arrow_length : $pad;
 }
 
@@ -43,14 +45,26 @@ sub draw_connectors {
     @parts = $self;
   }
 
-  if ($self->feature->strand >= 0) {
-    my($x1,$y1,$x2,$y2) = $parts[-1]->bounds(@_);
+  # flip argument makes this confusing
+  # certainly there's a simpler way to express this idea
+  my $strand    = $self->feature->strand;
+  my ($first,$last) = ($parts[0],$parts[-1]);
+  ($first,$last) = ($last,$first) if exists $self->{flip};
+
+  if ($strand >= 0) {
+    my($x1,$y1,$x2,$y2) = $last->bounds(@_);
     my $center = ($y2+$y1)/2;
-    $self->arrow($gd,$x2,$x2+$self->arrow_length,$center);
+    $self->{flip} ?
+	$self->arrow($gd,$x1,$x1-$self->arrow_length,$center)
+      :
+	$self->arrow($gd,$x2,$x2+$self->arrow_length,$center);
   } else {
-    my($x1,$y1,$x2,$y2) = $parts[0]->bounds(@_);
+    my($x1,$y1,$x2,$y2) = $first->bounds(@_);
     my $center = ($y2+$y1)/2;
-    $self->arrow($gd,$x1,$x1 - $self->arrow_length,$center);
+    $self->{flip } ?
+	$self->arrow($gd,$x2,$x2+$self->arrow_length,$center)
+      :
+	$self->arrow($gd,$x1,$x1 - $self->arrow_length,$center);
   }
 }
 
@@ -63,7 +77,7 @@ sub arrow_length {
 sub connector {
   my $self = shift;
   return $self->SUPER::connector(@_) if $self->all_callbacks;
-  return 'hat';
+  return ($self->option('connector') || 'hat');
 }
 
 
