@@ -1,6 +1,6 @@
-# $Id: EMBL.pm,v 1.19 2001/05/16 14:57:37 heikki Exp $
+# $Id$
 #
-# BioPerl module for Bio::Index::Abstract
+# BioPerl module for Bio::Index::EMBL
 #
 # Cared for by Ewan Birney <birney@sanger.ac.uk>
 #
@@ -18,15 +18,17 @@ Bio::Index::EMBL - Interface for indexing (multiple) EMBL/Swissprot
     # Complete code for making an index for several
     # EMBL files
     use Bio::Index::EMBL;
+    use strict;
 
     my $Index_File_Name = shift;
-    my $inx = Bio::Index::EMBL->new('-filename' => $Index_File_Name, 
+    my $inx = Bio::Index::EMBL->new('-filename' => $Index_File_Name,
 				    '-write_flag' => 'WRITE');
     $inx->make_index(@ARGV);
 
     # Print out several sequences present in the index
     # in Fasta format
     use Bio::Index::EMBL;
+    use strict;
 
     my $Index_File_Name = shift;
     my $inx = Bio::Index::EMBL->new('-filename' => $Index_File_Name);
@@ -40,14 +42,14 @@ Bio::Index::EMBL - Interface for indexing (multiple) EMBL/Swissprot
     # alternatively
 
     my $seq1 = $inx->get_Seq_by_id($id);
-    my $seq2 = $inx->get_Seq_by_acc($acc);   
+    my $seq2 = $inx->get_Seq_by_acc($acc);
 
 =head1 DESCRIPTION
 
 Inherits functions for managing dbm files from Bio::Index::Abstract.pm,
 and provides the basic funtionallity for indexing EMBL files, and
 retrieving the sequence from them. Heavily snaffled from James Gilbert's
-Fasta system.
+Fasta system. Note: for best results 'use strict'.
 
 =head1 FEED_BACK
 
@@ -69,7 +71,7 @@ the bugs and their resolution.  Bug reports can be submitted via
 email or the web:
 
   bioperl-bugs@bio.perl.org
-  http://bio.perl.org/bioperl-bugs/
+  http://bugzilla.bioperl.org/
 
 =head1 AUTHOR - Ewan Birney
 
@@ -130,7 +132,7 @@ sub _index_file {
         $file, # File name
         $i     # Index-number of file being indexed
         ) = @_;
-    
+
     my( $begin, # Offset from start of file of the start
                 # of the last found record.
         $id,    # ID of last found record.
@@ -165,10 +167,10 @@ sub _index_file {
 	    $id = $1;
 	    # not sure if I like this. Assummes tell is in bytes.
 	    # we could tell before each line and save it.
-            $begin = tell(EMBL) - length( $_ ); 
-	    
-	} elsif (/^AC(.*)/) { # ignore ? if there.
-	    @accs = ($1 =~ /\s*(\S+);/g);
+            $begin = tell(EMBL) - length( $_ );
+	
+	} elsif (/^AC\s+(.*)?/) {
+            push @accs , split (/[; ]+/, $1);
 	} else {
 	    # do nothing
 	}
