@@ -1,4 +1,4 @@
-# $Id: Exon.pm,v 1.4.2.5 2001/06/22 10:40:02 heikki Exp $
+# $Id$
 #
 # bioperl module for Bio::LiveSeq::Exon
 #
@@ -44,15 +44,15 @@ methods. Internal methods are usually preceded with a _
 # Let the code begin...
 
 package Bio::LiveSeq::Exon;
-$version=1.1;
+$VERSION=1.1;
 
 # Version history:
 # Mon Mar 20 22:26:13 GMT 2000 v 1.0 begun
 # Wed Apr 12 12:42:56 BST 2000 v 1.1 get_Transcript added
 
 use strict;
-use vars qw($version @ISA);
-use Bio::LiveSeq::Range; # uses Range, inherits from it
+use vars qw($VERSION @ISA);
+use Bio::LiveSeq::Range 1.2; # uses Range, inherits from it
 @ISA=qw(Bio::LiveSeq::Range);
 
 =head2 new
