@@ -1,5 +1,4 @@
-
-# $Id: Intron.pm,v 1.3.2.5 2001/06/22 10:40:02 heikki Exp $
+# $Id$
 #
 # bioperl module for Bio::LiveSeq::Intron
 #
@@ -45,14 +44,14 @@ methods. Internal methods are usually preceded with a _
 # Let the code begin...
 
 package Bio::LiveSeq::Intron;
-$version=1.0;
+$VERSION=1.0;
 
 # Version history:
 # Mon Mar 20 22:26:13 GMT 2000 v 1.0 begun
 
 use strict;
-use vars qw($version @ISA);
-use Bio::LiveSeq::Range; # uses Range, inherits from it
+use vars qw($VERSION @ISA);
+use Bio::LiveSeq::Range 1.2; # uses Range, inherits from it
 @ISA=qw(Bio::LiveSeq::Range);
 
 =head2 new
