@@ -1,4 +1,4 @@
-# $Id: SimpleOntologyEngine.pm,v 1.3.2.3 2003/03/27 10:07:56 lapp Exp $
+# $Id$
 #
 # BioPerl module for SimpleOntologyEngine
 #
@@ -256,8 +256,8 @@ sub get_term_by_identifier{
   my @ans = ();
 
   foreach my $id (@ids) {
-    my $term = $self->_term_store->{$id};
-    push @ans, $term if defined $term;
+      my $term = $self->_term_store->{$id};
+      push @ans, $term if defined $term;
   }
 
   return @ans;
@@ -398,24 +398,27 @@ sub _relationship_type_store{
 =cut
 
 sub _add_relationship_simple{
-   my ($self, $store, $relationship, $inverted) = @_;
+   my ($self, $store, $rel, $inverted) = @_;
    my $parent_id;
    my $child_id;
 
    if ($inverted) {
-     $parent_id = $relationship->subject_term->identifier;
-     $child_id = $relationship->object_term->identifier;
+     $parent_id = $rel->subject_term->identifier;
+     $child_id = $rel->object_term->identifier;
    }
    else {
-     $parent_id = $relationship->object_term->identifier;
-     $child_id = $relationship->subject_term->identifier;
+     $parent_id = $rel->object_term->identifier;
+     $child_id = $rel->subject_term->identifier;
    }
-   if ( defined $store->{$parent_id}->{$child_id} ) {
-     $self->throw("relationship ".Dumper($relationship->predicate_term)." between ".$parent_id." and ".$child_id." already defined as ".Dumper($store->{$parent_id}->{$child_id})."\n")
-       if ($store->{$parent_id}->{$child_id}->name != $relationship->predicate_term->name);
+   if((defined $store->{$parent_id}->{$child_id}) &&
+      ($store->{$parent_id}->{$child_id}->name != $rel->predicate_term->name)){
+       $self->throw("relationship ".Dumper($rel->predicate_term).
+		    " between ".$parent_id." and ".$child_id.
+		    " already defined as ".
+		    Dumper($store->{$parent_id}->{$child_id})."\n");
    }
    else {
-     $store->{$parent_id}->{$child_id} = $relationship->predicate_term;
+     $store->{$parent_id}->{$child_id} = $rel->predicate_term;
    }
 }
 
@@ -438,8 +441,8 @@ sub add_relationship{
 				   $rel, 0);
    $self->_add_relationship_simple($self->_inverted_relationship_store,
 				   $rel, 1);
-   $self->_relationship_type_store->{$rel->predicate_term->identifier} =
-       $rel->predicate_term;
+   $self->_relationship_type_store->{
+       $self->_unique_termid($rel->predicate_term)} = $rel->predicate_term;
 
    return $rel;
 }
@@ -561,7 +564,11 @@ sub _is_rel_type{
   my ($self, $term, @rel_types) = @_;
 
   foreach my $rel_type (@rel_types) {
-    return 1 if $rel_type->identifier eq $term->identifier;
+      if($rel_type->identifier || $term->identifier) {
+	  return 1 if $rel_type->identifier eq $term->identifier;
+      } else {
+	  return 1 if $rel_type->name eq $term->name;
+      }
   }
 
   return 0;
@@ -591,7 +598,9 @@ sub _typed_traversal{
     @ans = ();
 
     foreach my $child_id (@children) {
-      push @ans, $child_id if $self->_is_rel_type( $rel_store->{$term_id}->{$child_id}, @rel_types);
+      push @ans, $child_id
+	  if $self->_is_rel_type( $rel_store->{$term_id}->{$child_id},
+				  @rel_types);
     }
   }
   else {
@@ -601,7 +610,8 @@ sub _typed_traversal{
     my @ans1 = ();
 
     foreach my $child_id (@ans) {
-      push @ans1, $self->_typed_traversal($rel_store, $level - 1, $child_id, @rel_types)
+      push @ans1, $self->_typed_traversal($rel_store,
+					  $level - 1, $child_id, @rel_types)
 	if defined $rel_store->{$child_id};
     }
     push @ans, @ans1;
@@ -615,7 +625,12 @@ sub _typed_traversal{
  Title   : get_child_terms
  Usage   : get_child_terms(TermI term, TermI[] predicate_terms): TermI[]
   get_child_terms(TermI term, RelationshipType[] predicate_terms): TermI[]
- Function: Retrieves all child terms of a given term, that satisfy a relationship among those that are specified in the second argument or undef otherwise. get_child_terms is a special case of get_descendant_terms, limiting the search to the direct descendants.
+ Function: Retrieves all child terms of a given term, that satisfy a
+           relationship among those that are specified in the second
+           argument or undef otherwise. get_child_terms is a special
+           case of get_descendant_terms, limiting the search to the
+           direct descendants.
+
  Example :
  Returns : Array of TermI objects.
  Args    : First argument is the term of interest, second is the list of relationship type terms.
@@ -624,10 +639,17 @@ sub _typed_traversal{
 =cut
 
 sub get_child_terms{
-  my ($self, $term, @relationship_types) = @_;
+    my ($self, $term, @relationship_types) = @_;
 
-  die if !defined $term;
-  return $self->_filter_unmarked( $self->get_term_by_identifier( $self->_typed_traversal($self->_relationship_store, 1, $term->identifier, @relationship_types) ) );
+    $self->throw("must provide TermI compliant object") 
+	unless defined($term) && $term->isa("Bio::Ontology::TermI");
+
+    return $self->_filter_unmarked(
+               $self->get_term_by_identifier(
+		   $self->_typed_traversal($self->_relationship_store,
+					   1,
+					   $term->identifier,
+					   @relationship_types) ) );
 }
 
 =head2 get_descendant_terms
@@ -635,7 +657,11 @@ sub get_child_terms{
  Title   : get_descendant_terms
  Usage   : get_descendant_terms(TermI term, TermI[] rel_types): TermI[]
   get_child_terms(TermI term, RelationshipType[] predicate_terms): TermI[]
- Function: Retrieves all descendant terms of a given term, that satisfy a relationship among those that are specified in the second argument or undef otherwise. Uses _typed_traversal to find all descendants.
+ Function: Retrieves all descendant terms of a given term, that
+           satisfy a relationship among those that are specified in
+           the second argument or undef otherwise. Uses
+           _typed_traversal to find all descendants.
+
  Example :
  Returns : Array of TermI objects.
  Args    : First argument is the term of interest, second is the list of relationship type terms.
@@ -646,9 +672,16 @@ sub get_child_terms{
 sub get_descendant_terms{
   my ($self, $term, @relationship_types) = @_;
 
-  $self->throw("must provide TermI compliant object") unless defined $term;
+  $self->throw("must provide TermI compliant object") 
+      unless defined($term) && $term->isa("Bio::Ontology::TermI");
 
-  return $self->_filter_unmarked( $self->_filter_repeated( $self->get_term_by_identifier( $self->_typed_traversal($self->_relationship_store, 0, $term->identifier, @relationship_types) ) ) );
+  return $self->_filter_unmarked(
+	     $self->_filter_repeated(
+	         $self->get_term_by_identifier(
+		     $self->_typed_traversal($self->_relationship_store,
+					     0,
+					     $term->identifier,
+					     @relationship_types) ) ) );
 }
 
 =head2 get_parent_terms
@@ -656,7 +689,12 @@ sub get_descendant_terms{
  Title   : get_parent_terms
  Usage   : get_parent_terms(TermI term, TermI[] predicate_terms): TermI[]
   get_child_terms(TermI term, RelationshipType[] predicate_terms): TermI[]
- Function: Retrieves all parent terms of a given term, that satisfy a relationship among those that are specified in the second argument or undef otherwise. get_parent_terms is a special case of get_ancestor_terms, limiting the search to the direct ancestors.
+ Function: Retrieves all parent terms of a given term, that satisfy a
+           relationship among those that are specified in the second
+           argument or undef otherwise. get_parent_terms is a special
+           case of get_ancestor_terms, limiting the search to the
+           direct ancestors.
+
  Example :
  Returns : Array of TermI objects.
  Args    : First argument is the term of interest, second is the list of relationship type terms.
@@ -666,7 +704,7 @@ sub get_descendant_terms{
 
 sub get_parent_terms{
   my ($self, $term, @relationship_types) = @_;
-  die if !defined $term;
+  $self->throw("term must be a valid object, not undef") unless defined $term;
 
   return $self->_filter_unmarked(
 	    $self->get_term_by_identifier(
@@ -681,19 +719,30 @@ sub get_parent_terms{
  Title   : get_ancestor_terms
  Usage   : get_ancestor_terms(TermI term, TermI[] predicate_terms): TermI[]
   get_child_terms(TermI term, RelationshipType[] predicate_terms): TermI[]
- Function: Retrieves all ancestor terms of a given term, that satisfy a relationship among those that are specified in the second argument or undef otherwise. Uses _typed_traversal to find all ancestors.
+ Function: Retrieves all ancestor terms of a given term, that satisfy
+           a relationship among those that are specified in the second
+           argument or undef otherwise. Uses _typed_traversal to find
+           all ancestors.
+
  Example :
  Returns : Array of TermI objects.
- Args    : First argument is the term of interest, second is the list of relationship type terms.
+ Args    : First argument is the term of interest, second is the list
+           of relationship type terms.
 
 
 =cut
 
 sub get_ancestor_terms{
   my ($self, $term, @relationship_types) = @_;
-  die if !defined $term;
+  $self->throw("term must be a valid object, not undef") unless defined $term;
 
-  return $self->_filter_unmarked( $self->_filter_repeated( $self->get_term_by_identifier( $self->_typed_traversal($self->_inverted_relationship_store, 0, $term->identifier, @relationship_types) ) ) );
+  return $self->_filter_unmarked(
+	    $self->_filter_repeated(
+               $self->get_term_by_identifier(
+                  $self->_typed_traversal($self->_inverted_relationship_store,
+					  0,
+					  $term->identifier,
+					  @relationship_types) ) ) );
 }
 
 =head2 get_leaf_terms
@@ -907,7 +956,9 @@ sub _filter_unmarked{
 
  Title   : remove_term_by_id
  Usage   : remove_term_by_id(String id): TermI
- Function: Removes TermI object from the ontology engine using the string id as an identifier. Current implementation does not enforce consistency of the relationships using that term.
+ Function: Removes TermI object from the ontology engine using the
+           string id as an identifier. Current implementation does not
+           enforce consistency of the relationships using that term.
  Example : $term = $soe->remove_term_by_id($id);
  Returns : Object of class TermI or undef if not found.
  Args    : The string identifier of a term.
@@ -924,7 +975,7 @@ sub remove_term_by_id{
     return $term;
   }
   else {
-    warn "Term with id: ".$id." is not in the term store.\n";
+    $self->warn("Term with id '$id' is not in the term store");
     return undef;
   }
 }
@@ -933,7 +984,8 @@ sub remove_term_by_id{
 
  Title   : to_string
  Usage   : print $sv->to_string();
- Function: to_string method for simple ontology engine. Currently returns formatted string containing the number of terms and number of relationships from the ontology engine.
+ Function: Currently returns formatted string containing the number of
+           terms and number of relationships from the ontology engine.
  Example : print $sv->to_string();
  Returns :
  Args    :
@@ -946,13 +998,49 @@ sub to_string{
   my $s = "";
 
   $s .= "-- # Terms:\n";
-  $s .= (scalar $self->get_all_terms)."\n";
+  $s .= scalar($self->get_all_terms)."\n";
   $s .= "-- # Relationships:\n";
   $s .= $self->_get_number_rels."\n";
 
   return $s;
 }
 
+=head2 _unique_termid
+
+ Title   : _unique_termid
+ Usage   :
+ Function: Returns a string that can be used as ID using fail-over
+           approaches. 
+
+           If the identifier attribute is not set, it uses the
+           combination of name and ontology name, provided both are
+           set. If they aren't, it returns the name alone.
+
+           Note that this is a private method. Call from inheriting
+           classes but not from outside.
+
+ Example :
+ Returns : a string
+ Args    : a Bio::Ontology::TermI compliant object
+
+
+=cut
+
+sub _unique_termid{
+    my $self = shift;
+    my $term = shift;
+
+    return $term->identifier() if $term->identifier();
+    my $id = $term->ontology->name() if $term->ontology();
+    if($id) { 
+	$id .= '|'; 
+    } else { 
+	$id = ''; 
+    }
+    $id .= $term->name();
+}
+
+
 #################################################################
 # aliases
 #################################################################
