@@ -2,9 +2,9 @@
 # PACKAGE : Bio::Root::Global.pm
 # PURPOSE : Provides global data, objects, and methods potentially useful to 
 #           many different modules and scripts.
-# AUTHOR  : Steve A. Chervitz (sac@genome.stanford.edu)
+# AUTHOR  : Steve Chervitz (sac@bioperl.org)
 # CREATED : 3 Sep 1996
-# REVISION: $Id: Global.pm,v 1.4 1999/04/06 11:14:11 sac Exp $
+# REVISION: $Id$
 #
 # INSTALLATION:
 #   This module is included with the central Bioperl distribution:
@@ -28,7 +28,40 @@
 #    sac --- Fri 5 Jun 1998: Added @DAYS.
 #    sac --- Sun Aug 16 1998: Added $RECORD_ERR and &record_err().
 #--------------------------------------------------------------------------------
+
+### POD Documentation:
+
+=head1 NAME
+
+Bio::Root::Global - Global variables and utility functions
+
+=head1 SYNOPSIS
+
+    # no real synopsis - see Bio::Root::Object
+
+=head1 DESCRIPTION
+
+The Bio::Root::Global file contains all the global flags
+about erro warning etc, and also utility functions, eg
+to map numbers to roman numerals.
+
+These functions are generally called by Bio::Root::Object
+or somewhere similar, and not directly
+
+
+=head1 INSTALLATION
+
+This module is included with the central Bioperl distribution:
+
+   http://bio.perl.org/Core/Latest
+   ftp://bio.perl.org/pub/DIST
+
+Follow the installation instructions included in the README file.
+
+=cut
+
 package	 Bio::Root::Global;
+use strict;
 
 BEGIN {
     use vars qw($CGI $TIMEOUT_SECS);
@@ -43,6 +76,7 @@ BEGIN {
 use Exporter ();
 use vars qw($BASE_YEAR @DAYS @MONTHS);
 
+use vars qw(@ISA @EXPORT_OK %EXPORT_TAGS);
 @ISA       = qw( Exporter );
 @EXPORT_OK = qw($AUTHORITY $NEWLINE
 		$DEBUG $MONITOR $TESTING 
@@ -84,8 +118,12 @@ use vars qw($BASE_YEAR @DAYS @MONTHS);
 ##             Data                 ##
 ######################################
 
+use vars qw($AUTHORITY $DEBUG $MONITOR $TESTING $DONT_WARN $WARN_ON_FATAL
+            $FATAL_ON_WARN $RECORD_ERR $STRICTNESS $VERBOSITY $NEWLINE
+            %ROMAN_NUMS $GLOBAL);
+
 # Who should receive feedback from users and possibly automatic error messages.
-$AUTHORITY     = 'sac@genome.stanford.edu';
+$AUTHORITY     = 'sac@bioperl.org';
  
 $DEBUG         = 0;
 $MONITOR       = 0;
@@ -215,3 +253,9 @@ sub fatal_on_warn {
 #####################################################################################
 
 1;
+
+
+
+
+
+
