@@ -1,5 +1,5 @@
 #-----------------------------------------------------------------
-# $Id: BlastHSP.pm,v 1.11.2.2 2002/05/30 03:18:04 jason Exp $
+# $Id$
 #
 # BioPerl module Bio::Search::HSP::BlastHSP
 #
@@ -97,7 +97,7 @@ the bugs and their resolution. Bug reports can be submitted via email
 or the web:
 
     bioperl-bugs@bio.perl.org                   
-    http://bio.perl.org/bioperl-bugs/           
+    http://bugzilla.bioperl.org/     
 
 =head1 AUTHOR 
 
@@ -159,7 +159,7 @@ use vars qw( @ISA $GAP_SYMBOL $Revision %STRAND_SYMBOL );
 use overload 
     '""' => \&to_string;
 
-$Revision = '$Id: BlastHSP.pm,v 1.11.2.2 2002/05/30 03:18:04 jason Exp $';  #'
+$Revision = '$Id$';  #'
 
 @ISA = qw(Bio::SeqFeature::SimilarityPair Bio::Search::HSP::HSPI);
 
@@ -186,8 +186,6 @@ $GAP_SYMBOL    = '-';          # Need a more general way to handle gap symbols.
            :      -PROGRAM      => string ('TBLASTN', 'BLASTP', etc.).
            :      -QUERY_NAME   => string, id of query sequence
            :      -HIT_NAME     => string, id of hit sequence
-           :      -QUERY_LENGTH => integer, length of query sequence
-           :      -HIT_LENGTH   => integer, length of hit sequence
            :
  Comments  : Having the raw data allows this object to do lazy parsing of 
            : the raw HSP data (i.e., not parsed until needed).
@@ -208,43 +206,46 @@ sub new {
     my ($class, @args ) = @_;
 
     my $self = $class->SUPER::new( @args );
-
+    # Initialize placeholders
+    $self->{'_queryGaps'} = $self->{'_sbjctGaps'} = 0;
     my ($raw_data, $qname, $hname, $qlen, $hlen);
 
     ($self->{'_prog'}, $self->{'_rank'}, $raw_data,
-     $qname, $hname, $qlen, $hlen) = 
+     $qname, $hname) = 
       $self->_rearrange([qw( PROGRAM
 			     RANK
 			     RAW_DATA
 			     QUERY_NAME
 			     HIT_NAME
-			     QUERY_LENGTH
-			     HIT_LENGTH
 			   )], @args );
     
     # _set_data() does a fair amount of parsing. 
     # This will likely change (see comment above.)
     $self->_set_data( @{$raw_data} );
-
     # Store the aligned query as sequence feature
-    my ($qb, $hb) = $self->start();
-    my ($qe, $he) = $self->end();
-    my ($qs, $hs) = $self->strand();
+    my ($qb, $hb) = ($self->start());
+    my ($qe, $he) = ($self->end());
+    my ($qs, $hs) = ($self->strand());
+    my ($qf,$hf) = ($self->query->frame(),
+		    $self->hit->frame);
 
-    $self->query( Bio::SeqFeature::Similarity->new (-start=>$qb, 
-						    -end=>$qe, 
-						    -strand=>$qs, 
-						    -source=>$self->{'_prog'} ));
-
-    $self->hit( Bio::SeqFeature::Similarity->new (-start=>$hb, 
-						  -end=>$he, 
-						  -strand=>$hs, 
-						  -source=>$self->{'_prog'} ));
-
-
-    # name the sequences
-    $self->query->seqname($qname); # query
-    $self->hit->seqname($hname); # subject
+    $self->query( Bio::SeqFeature::Similarity->new (-start   =>$qb, 
+						    -end     =>$qe, 
+						    -strand  =>$qs, 
+						    -bits    =>$self->bits,
+						    -score   =>$self->score, 
+						    -frame   =>$qf,
+						    -seq_id  => $qname,
+						    -source  =>$self->{'_prog'} ));
+    
+    $self->hit( Bio::SeqFeature::Similarity->new (-start   =>$hb, 
+						  -end     =>$he, 
+						  -strand  =>$hs, 
+						  -bits    =>$self->bits,
+						  -score   =>$self->score,
+                                                  -frame   =>$hf, 
+						  -seq_id  => $hname,
+						  -source  =>$self->{'_prog'} ));
 
     # set lengths
     $self->query->seqlength($qlen); # query
@@ -252,13 +253,28 @@ sub new {
 
     $self->query->frac_identical($self->frac_identical('query'));
     $self->hit->frac_identical($self->frac_identical('hit'));
-
     return $self;
 }
 
-sub DESTROY {
-    my $self = shift; 
-    #print STDERR "--->DESTROYING $self\n";
+#sub DESTROY {
+#    my $self = shift; 
+#    #print STDERR "--->DESTROYING $self\n";
+#}
+
+
+# Title   : _id_str; 
+# Purpose : Intended for internal use only to provide a string for use
+#           within exception messages to help users figure out which 
+#           query/hit caused the problem.
+# Returns : Short string with name of query and hit seq 
+sub _id_str {
+    my $self = shift;
+    if( not defined $self->{'_id_str'}) {
+        my $qname = $self->query->seqname;
+        my $hname = $self->hit->seqname;
+        $self->{'_id_str'} = "QUERY=\"$qname\" HIT=\"$hname\"";
+    }
+    return $self->{'_id_str'};
 }
 
 #=================================================
@@ -419,18 +435,18 @@ See Also   : L<length()|length>, L<matches()|matches>
 sub gaps {
 #---------
     my( $self, $seqType ) = @_;
-
+    
     $self->_set_seq_data() unless $self->{'_set_seq_data'};
 
     $seqType  ||= (wantarray ? 'list' : 'total');
     $seqType = 'sbjct' if $seqType eq 'hit';
-
+    
     if($seqType =~ /list|array/i) {
 	return (($self->{'_queryGaps'} || 0), ($self->{'_sbjctGaps'} || 0));
     }
-
+    
     if($seqType eq 'total') {
-	return (($self->{'_queryGaps'} || 0) + ($self->{'_sbjctGaps'} || 0)) || 0;
+	return ($self->{'_queryGaps'} + $self->{'_sbjctGaps'}) || 0;
     } else {
 	## Sensitive to member name format.
 	$seqType = "_\L$seqType\E";
@@ -716,7 +732,6 @@ sub _set_data {
 	    push @sbjctList, $line;
 	}
     }
-
     # Storing the query and sbjct lists in case they are needed later.
     # We could make this conditional to save memory.
     $self->{'_queryList'} = \@queryList; 
@@ -726,12 +741,14 @@ sub _set_data {
     $self->{'_matchList'} = \@matchList;
 
     if(not defined ($self->{'_numIdentical'})) {
-      $self->throw( -text  => "Can't parse match statistics. Possibly a new or unrecognized Blast format.");
+        my $id_str = $self->_id_str;
+        $self->throw( -text  => "Can't parse match statistics. Possibly a new or unrecognized Blast format. ($id_str)");
     }
 
     if(!scalar @queryList or !scalar @sbjctList) {
-        $self->throw( "Can't find query or sbjct alignment lines. Possibly unrecognized Blast format.");
-      }
+        my $id_str = $self->_id_str;
+        $self->throw( "Can't find query or sbjct alignment lines. Possibly unrecognized Blast format. ($id_str)");
+    }
 }
 
 
@@ -786,17 +803,16 @@ sub _set_score_stats {
 	$p                 = $5;
 
     } else {
+        my $id_str = $self->_id_str;
 	$self->throw(-class => 'Bio::Root::Exception',
-		     -text => "Can't parse score statistics: unrecognized format.", 
+		     -text => "Can't parse score statistics: unrecognized format. ($id_str)", 
 		     -value => $data);
     }
-
     $expect = "1$expect" if $expect =~ /^e/i;    
     $p      = "1$p"      if defined $p and $p=~ /^e/i; 
 
     $self->{'_expect'} = $expect;
-    $self->{'_p'}      = $p || undef;
-
+    $self->{'_p'}      = $p || undef;    
     $self->significance( $p || $expect );
 }
 
@@ -927,16 +943,19 @@ sub _set_seq {
     my $numGaps   = 0;
 
     foreach( @data ) {
-	if( m/(\d+) *(\D+) *(\d+)/) {
-	    push @ranges, ( $1, $3 ) ;
-	    push @sequence, $2;
+        if( m/(\d+) *([^\d\s]+) *(\d+)/) {
+            push @ranges, ( $1, $3 ) ;
+            push @sequence, $2;
+        #print STDERR "_set_seq found sequence \"$2\"\n";
 	} else {
 	    $self->warn("Bad sequence data: $_");
 	}
     }
 
-    (scalar(@sequence) and scalar(@ranges)) ||
-	$self->throw("Can't set sequence: missing data. Possibly unrecognized Blast format.");
+    if( !(scalar(@sequence) and scalar(@ranges))) {
+        my $id_str = $self->_id_str;
+	$self->throw("Can't set sequence: missing data. Possibly unrecognized Blast format. ($id_str)");
+   }
  
     # Sensitive to member name changes.
     $seqType = "_\L$seqType\E";
@@ -949,7 +968,7 @@ sub _set_seq {
     # Adjust lengths for BLASTX, TBLASTN, TBLASTX sequences
     # Converting nucl coords to amino acid coords.
 
-    my $prog = $self->{'_prog'};
+    my $prog = $self->algorithm;
     if($prog eq 'TBLASTN' and $seqType eq '_sbjct') {
 	$self->{$seqType.'Length'} /= 3;
     } elsif($prog eq 'BLASTX' and $seqType eq '_query') {
@@ -1019,7 +1038,7 @@ sub _set_residues {
     my $resCount_query = $self->{'_queryStop'} || 0;
     my $resCount_sbjct = $self->{'_sbjctStop'} || 0;
 
-    my $prog = $self->{'_prog'};
+    my $prog = $self->algorithm;
     if($prog !~ /^BLASTP|^BLASTN/) {
 	if($prog eq 'TBLASTN') {
 	    $resCount_sbjct /= 3;
@@ -1075,7 +1094,10 @@ sub _set_match_seq {
 #-------------------
     my $self = shift;
 
-    ref($self->{'_matchList'}) || $self->throw("Can't set HSP match sequence: No data");
+    if( ! ref($self->{'_matchList'}) ) {
+        my $id_str = $self->_id_str;
+        $self->throw("Can't set HSP match sequence: No data ($id_str)");
+    }
     
     my @data = @{$self->{'_matchList'}};
 
@@ -1151,7 +1173,9 @@ sub matches {
     my(@data);
     my($seqType, $beg, $end) = ($param{-SEQ}, $param{-START}, $param{-STOP});
     $seqType ||= 'query';
-   $seqType = 'sbjct' if $seqType eq 'hit';
+    $seqType = 'sbjct' if $seqType eq 'hit';
+
+    my($start,$stop);
 
     if(!defined $beg && !defined $end) {
 	## Get data for the whole alignment.
@@ -1160,7 +1184,7 @@ sub matches {
 	## Get the substring representing the desired sub-section of aln.
 	$beg ||= 0;
 	$end ||= 0;
-	my($start,$stop) = $self->range($seqType);
+	($start,$stop) = $self->range($seqType);
 	if($beg == 0) { $beg = $start; $end = $beg+$end; }
 	elsif($end == 0) { $end = $stop; $beg = $end-$beg; }
 
@@ -1175,12 +1199,13 @@ sub matches {
 
 	## ML: START fix for substr out of range error ------------------
 	my $seq = "";
-	if (($self->{'_prog'} eq 'TBLASTN') and ($seqType eq 'sbjct'))
+        my $prog = $self->algorithm;
+	if (($prog eq 'TBLASTN') and ($seqType eq 'sbjct'))
 	{
 	    $seq = substr($self->seq_str('match'),
 			  int(($beg-$start)/3), int(($end-$beg+1)/3));
 
-	} elsif (($self->{'_prog'} eq 'BLASTX') and ($seqType eq 'query'))
+	} elsif (($prog eq 'BLASTX') and ($seqType eq 'query'))
 	{
 	    $seq = substr($self->seq_str('match'),
 			  int(($beg-$start)/3), int(($end-$beg+1)/3));
@@ -1205,7 +1230,8 @@ sub matches {
 	 ## ML: END DEBUGGING CODE----------
 
 	if(!CORE::length $seq) {
-	    $self->throw("Undefined sub-sequence ($beg,$end). Valid range = $start - $stop");
+            my $id_str = $self->_id_str;
+	    $self->throw("Undefined $seqType sub-sequence ($beg,$end). Valid range = $start - $stop ($id_str)");
 	}
 	## Get data for a substring.
 #	printf "Collecting HSP subsection data: beg,end = %d,%d; start,stop = %d,%d\n%s<---\n", $beg, $end, $start, $stop, $seq;
@@ -1422,7 +1448,7 @@ sub strand {
     # $seqType could be '_list'.
     $self->{'_queryStrand'} or $self->_set_seq_data() unless $self->{'_set_seq_data'};
 
-    my $prog = $self->{'_prog'};
+    my $prog = $self->algorithm;
 
     if($seqType  =~ /list|array/i) {
         my ($qstr, $hstr);
@@ -1442,7 +1468,8 @@ sub strand {
             $qstr = $STRAND_SYMBOL{$self->{'_queryStrand'}} if defined $self->{'_queryStrand'};
             $hstr = $STRAND_SYMBOL{$self->{'_sbjctStrand'}} if defined $self->{'_sbjctStrand'};
         }
-	return ($qstr, $hstr);
+        $qstr ||= 0;
+        $hstr ||= 0;  
 	return ($qstr, $hstr);
     }
     local $^W = 0;
@@ -1527,8 +1554,9 @@ sub seq_str {
 	return join('',@$aref); 
 
     } else {
+        my $id_str = $self->_id_str;
 	$self->throw(-class => 'Bio::Root::BadParameter',
-		     -text => "Invalid or undefined sequence type: $seqType\n" . 
+		     -text => "Invalid or undefined sequence type: $seqType ($id_str)\n" . 
 		               "Valid types: query, sbjct, match",
 		     -value => $seqType);
     }
@@ -1601,9 +1629,7 @@ sub seq_inds {
  Comments  : Requires Bio::SimpleAlign.
            : The Bio::SimpleAlign object is constructed from the query + sbjct 
            : sequence objects obtained by calling seq().
-           : Gap residues are included (see $GAP_SYMBOL). It is important that 
-           : Bio::UnivAln recognizes the gaps correctly. A strategy for doing 
-           : this is being considered. Currently it is hard-wired.
+           : Gap residues are included (see $GAP_SYMBOL). 
 
 See Also   : L<seq()|seq>, L<Bio::SimpleAlign>
 
@@ -1619,7 +1645,7 @@ sub get_aln {
     my $qseq = $self->seq('query');
     my $sseq = $self->seq('sbjct');
 
-    my $type = $self->{'_prog'} =~ /P$|^T/ ? 'amino' : 'dna';
+    my $type = $self->algorithm =~ /P$|^T/ ? 'amino' : 'dna';
     my $aln = new Bio::SimpleAlign();
     $aln->add_seq(new Bio::LocatableSeq(-seq => $qseq->seq(),
 					-id  => 'query_'.$qseq->display_id(),
