@@ -2,7 +2,7 @@
 # PACKAGE : Bio::SeqIO::raw
 # AUTHOR  : Ewan Birney <birney@ebi.ac.uk>
 # CREATED : Feb 16 1999
-# REVISION: $Id: raw.pm,v 1.8 2001/10/31 13:11:47 avc Exp $
+# REVISION: $Id$
 #            
 # Copyright (c) 1997-9 bioperl, Ewan Birney. All Rights Reserved.
 #           This module is free software; you can redistribute it and/or 
@@ -59,12 +59,16 @@ Report bugs to the Bioperl bug tracking system to help us keep track
  Bug reports can be submitted via email or the web:
 
   bioperl-bugs@bio.perl.org
-  http://bio.perl.org/bioperl-bugs/
+  http://bugzilla.bioperl.org/
 
 =head1 AUTHORS
 
-  Ewan Birney  <birney@ebi.ac.uk>
-  Lincoln Stein <lstein@cshl.org>
+  Ewan Birney   E<lt>birney@ebi.ac.ukE<gt>
+  Lincoln Stein E<lt>lstein@cshl.orgE<gt>
+
+=head1 CONTRIBUTORS
+
+  Jason Stajich E<lt>jason@bioperl.org<gt>
 
 =head1 APPENDIX
 
@@ -81,10 +85,20 @@ use strict;
 use vars qw(@ISA);
 
 use Bio::SeqIO;
-use Bio::Seq;
+use Bio::Seq::SeqFactory;
 
 @ISA = qw(Bio::SeqIO);
 
+sub _initialize {
+  my($self,@args) = @_;
+  $self->SUPER::_initialize(@args);    
+  if( ! defined $self->sequence_factory ) {
+      $self->sequence_factory(new Bio::Seq::SeqFactory
+			      (-verbose => $self->verbose(), 
+			       -type => 'Bio::Seq'));      
+  }
+}
+
 =head2 next_seq
 
  Title   : next_seq
@@ -107,8 +121,7 @@ sub next_seq{
    my $sequence = uc($nextline);
    $sequence =~ s/\W//g;
 
-   return  Bio::Seq->new(-seq => $sequence);
-
+   return  $self->sequence_factory->create(-seq => $sequence);
 }
 
 =head2 write_seq
@@ -117,16 +130,19 @@ sub next_seq{
  Usage   : $stream->write_seq($seq)
  Function: writes the $seq object into the stream
  Returns : 1 for success and 0 for error
- Args    : Bio::Seq object
+ Args    : Array of Bio::PrimarySeqI objects
 
 
 =cut
 
 sub write_seq {
    my ($self,@seq) = @_;
-   foreach (@seq) {
-     $self->_print($_->seq, "\n") or return;
+   foreach my $seq (@seq) {
+       $self->throw("Must provide a valid Bio::PrimarySeqI object") 
+	   unless defined $seq && ref($seq) && $seq->isa('Bio::PrimarySeqI');
+     $self->_print($seq->seq, "\n") or return;
    }
+   $self->flush if $self->_flush_on_write && defined $self->_fh;
    return 1;
 }
 
