@@ -1,4 +1,4 @@
-# $Id: scf.pm,v 1.14 2002/02/14 16:01:45 matsallac Exp $
+# $Id$
 #
 # Copyright (c) 1997-2001 bioperl, Chad Matsalla. All Rights Reserved.
 #           This module is free software; you can redistribute it and/or
@@ -44,7 +44,7 @@ the bugs and their resolution.  Bug reports can be submitted via email
 or the web:
 
   bioperl-bugs@bio.perl.org
-  http://bio.perl.org/bioperl-bugs/
+  http://bugzilla.bioperl.org/
 
 =head1 AUTHOR Chad Matsalla
 
@@ -54,6 +54,8 @@ bioinformatics@dieselwurks.com
 =head1 CONTRIBUTORS
 
 Jason Stajich, jason@bioperl.org
+Tony Cox, avc@sanger.ac.uk
+Heikki Lehvaslaiho, heikki@ebi.ac.uk
 
 =head1 APPENDIX
 
@@ -68,9 +70,7 @@ package Bio::SeqIO::scf;
 use vars qw(@ISA $DEFAULT_QUALITY);
 use strict;
 use Bio::SeqIO;
-use Bio::Seq::PrimaryQual;
-use Bio::PrimarySeq;
-use Bio::Seq::SeqWithQuality;
+use Bio::Seq::SeqFactory;
 require 'dumpvar.pl';
 
 BEGIN { 
@@ -79,13 +79,23 @@ BEGIN {
 
 @ISA = qw(Bio::SeqIO);
 
+sub _initialize {
+  my($self,@args) = @_;
+  $self->SUPER::_initialize(@args);
+  if( ! defined $self->sequence_factory ) {
+      $self->sequence_factory(new Bio::Seq::SeqFactory
+			      (-verbose => $self->verbose(), 
+			       -type => 'Bio::Seq::SeqWithQuality'));
+  }
+}
+
 =head2 next_seq()
 
  Title   : next_seq()
  Usage   : $scf = $stream->next_seq()
  Function: returns the next scf sequence in the stream
  Returns : Bio::Seq::SeqWithQuality object
- Args    : NONE (huh?)
+ Args    : NONE
  Notes   : Fills the interface specification for SeqIO.
 	   The SCF specification does not provide for having more then
            one sequence in a given scf. So once the filehandle has been open
@@ -97,28 +107,10 @@ BEGIN {
 
 #'
 sub next_seq {
-	my ($self,@args) = @_;
-	my $swq = $self->_next_scf(@args);
-	return $swq;
-}
-
-=head2 _next_scf()
-
- Title   : _next_scf()
- Usage   : $scf = $stream->_next_scf() (but do not do this. use
-	   next_seq() instead.)
- Function: returns the next scf sequence in the stream
- Returns : Bio::Seq::SeqWithQuality object
- Args    : NONE (huh?)
- Notes   : An internal method. See next_seq for proper usage.
-
-=cut
-
-sub _next_scf {
-    my( $self, @args ) = @_;
+    my ($self) = @_;
     my ($seq, $seqc, $fh, $buffer, $offset, $length, $read_bytes, @read,
 	%names);
-		# set up a filehandle to read in the scf    
+    # set up a filehandle to read in the scf
     $fh = $self->_filehandle();
     unless ($fh) {		# simulate the <> function
 	if ( !fileno(ARGV) or eof(ARGV) ) {
@@ -130,75 +122,84 @@ sub _next_scf {
     }
     binmode $fh;		# for the Win32/Mac crowds
     return unless read $fh, $buffer, 128; # no exception; probably end of file
-		# the first thing to do is parse the header. This is common
-		# among all versions of scf.
+    # the first thing to do is parse the header. This is common
+    # among all versions of scf.
     $self->_set_header($buffer);
-		# the rest of the the information is different between the
-		# the different versions of scf.
-	if ($self->{'version'} lt "3.00") {
-			# first gather the trace information
-		$length = $self->{'samples'}*$self->{sample_size}*4;
-		$buffer = $self->read_from_buffer($fh,$buffer,$length);
-		@read = unpack "n$length",$buffer;
-			# these traces need to be split
-		$self->_set_v2_traces(\@read);
-			# now go and get the base information
-		$offset = $self->{bases_offset};
-    		$length = ($self->{bases} * 12);
-		seek $fh,$offset,0;
-		$buffer = $self->read_from_buffer($fh,$buffer,$length);
-			# now distill the information into its fractions.
-		$self->_set_v2_bases($buffer);
+    # the rest of the the information is different between the
+    # the different versions of scf.
+    my $byte = "n";
+    if ($self->{'version'} lt "3.00") {
+	# first gather the trace information
+	$length = $self->{'samples'}*$self->{sample_size}*4;
+	$buffer = $self->read_from_buffer($fh,$buffer,$length);
+	if ($self->{sample_size} == 1) {
+	    $byte = "c";
 	}
-	else {
-		my $transformed_read;
-		foreach (qw(A C G T)) {
-			$length = $self->{'samples'}*$self->{sample_size};
-			$buffer = $self->read_from_buffer($fh,$buffer,$length);
-			@read = unpack "n$length",$buffer;
-				# this little spurt of nonsense is because
-				# the trace values are given in the binary
-				# file as unsigned shorts but they really
-				# are signed. 30000 is an arbitrary number
-				# (will there be any traces with a given
-				# point greater then 30000? I hope not.
-				# once the read is read, it must be changed
-				# from relative 
-			for (my $element=0; $element < scalar(@read); $element++) {
-				if ($read[$element] > 30000) {
-					$read[$element] = $read[$element] - 65536;
-				}
-			}
-			$transformed_read = $self->_delta(\@read,"backward");
-			$self->{'traces'}->{$_} = join(' ',@{$transformed_read});
+	@read = unpack "${byte}${length}",$buffer;
+	# these traces need to be split
+	$self->_set_v2_traces(\@read);
+	# now go and get the base information
+	$offset = $self->{bases_offset};
+	$length = ($self->{bases} * 12);
+	seek $fh,$offset,0;
+	$buffer = $self->read_from_buffer($fh,$buffer,$length);
+	# now distill the information into its fractions.
+	$self->_set_v2_bases($buffer);
+    } else {
+	my $transformed_read;
+	foreach (qw(A C G T)) {
+	    $length = $self->{'samples'}*$self->{sample_size};
+	    $buffer = $self->read_from_buffer($fh,$buffer,$length);
+            if ($self->{sample_size} == 1) {
+                $byte = "c";
+            }
+	    @read = unpack "${byte}${length}",$buffer;
+	    # this little spurt of nonsense is because
+	    # the trace values are given in the binary
+	    # file as unsigned shorts but they really
+	    # are signed. 30000 is an arbitrary number
+	    # (will there be any traces with a given
+	    # point greater then 30000? I hope not.
+	    # once the read is read, it must be changed
+	    # from relative
+	    for (my $element=0; $element < scalar(@read); $element++) {
+		if ($read[$element] > 30000) {
+		    $read[$element] = $read[$element] - 65536;
 		}
-			# now go and get the peak index information
-		$offset = $self->{bases_offset};
-    		$length = ($self->{bases} * 4);
-		seek $fh,$offset,0;
-		$buffer = $self->read_from_buffer($fh,$buffer,$length);
-		$self->_set_v3_peak_indices($buffer);
-			# now go and get the accuracy information
-		$buffer = $self->read_from_buffer($fh,$buffer,$length);
-		$self->_set_v3_base_accuracies($buffer);
-			# OK, now go and get the base information.
-		$length = $self->{bases};
-		$buffer = $self->read_from_buffer($fh,$buffer,$length);
-		$self->{'parsed'}->{'sequence'} = unpack("a$length",$buffer);
-			# now, finally, extract the calls from the accuracy information.
-		$self->_set_v3_quality($self);
+	    }
+	    $transformed_read = $self->_delta(\@read,"backward");
+	    $self->{'traces'}->{$_} = join(' ',@{$transformed_read});
 	}
-		    # now go and get the comment information
-		$length = $self->{comment_size};
-		$buffer = $self->read_from_buffer($fh,$buffer,$length);
-		$self->_set_comments($buffer);
-	    my $swq = Bio::Seq::SeqWithQuality->new
-		(-seq  =>	$self->{'parsed'}->{'sequence'},
-		 -qual =>	$self->{'parsed'}->{'qualities'},
-		 -id	=>	$self->{'comments'}->{'NAME'});
-	    return $swq;
+	# now go and get the peak index information
+	$offset = $self->{bases_offset};
+	$length = ($self->{bases} * 4);
+	seek $fh,$offset,0;
+	$buffer = $self->read_from_buffer($fh,$buffer,$length);
+	$self->_set_v3_peak_indices($buffer);
+	# now go and get the accuracy information
+	$buffer = $self->read_from_buffer($fh,$buffer,$length);
+	$self->_set_v3_base_accuracies($buffer);
+	# OK, now go and get the base information.
+	$length = $self->{bases};
+	$buffer = $self->read_from_buffer($fh,$buffer,$length);
+	$self->{'parsed'}->{'sequence'} = unpack("a$length",$buffer);
+	# now, finally, extract the calls from the accuracy information.
+	$self->_set_v3_quality($self);
+    }
+    # now go and get the comment information
+    $offset = $self->{comments_offset};
+    seek $fh,$offset,0;
+    $length = $self->{comment_size};
+    $buffer = $self->read_from_buffer($fh,$buffer,$length);
+    $self->_set_comments($buffer);
+    return $self->sequence_factory->create
+	(-seq  =>	$self->{'parsed'}->{'sequence'},
+	 -qual =>	$self->{'parsed'}->{'qualities'},
+	 -id   =>	$self->{'comments'}->{'NAME'}
+	 );
 }
 
+
 =head2 _set_v3_quality()
 
  Title   : _set_v3_quality()
@@ -212,19 +213,19 @@ sub _next_scf {
 
 #'
 sub _set_v3_quality {
-	my $self = shift;
-	my @bases = split//,$self->{'parsed'}->{'sequence'};
-	my (@qualities,$currbase,$currqual,$counter);
-	for ($counter=0; $counter <= $#bases ; $counter++) {
-		$currbase = uc($bases[$counter]);
-		if ($currbase eq "A") { $currqual = $self->{'parsed'}->{'base_accuracies'}->{'A'}->[$counter]; }
-		elsif ($currbase eq "C") { $currqual = $self->{'parsed'}->{'base_accuracies'}->{'C'}->[$counter]; }
-		elsif ($currbase eq "G") { $currqual = $self->{'parsed'}->{'base_accuracies'}->{'G'}->[$counter]; }
-		elsif ($currbase eq "T") { $currqual = $self->{'parsed'}->{'base_accuracies'}->{'T'}->[$counter]; }
-		else { $currqual = "unknown"; }
-		push @qualities,$currqual;
-	}
-	$self->{'parsed'}->{'qualities'} = \@qualities;
+    my $self = shift;
+    my @bases = split//,$self->{'parsed'}->{'sequence'};
+    my (@qualities,$currbase,$currqual,$counter);
+    for ($counter=0; $counter <= $#bases ; $counter++) {
+	$currbase = uc($bases[$counter]);
+	if ($currbase eq "A") { $currqual = $self->{'parsed'}->{'base_accuracies'}->{'A'}->[$counter]; }
+	elsif ($currbase eq "C") { $currqual = $self->{'parsed'}->{'base_accuracies'}->{'C'}->[$counter]; }
+	elsif ($currbase eq "G") { $currqual = $self->{'parsed'}->{'base_accuracies'}->{'G'}->[$counter]; }
+	elsif ($currbase eq "T") { $currqual = $self->{'parsed'}->{'base_accuracies'}->{'T'}->[$counter]; }
+	else { $currqual = "unknown"; }
+	push @qualities,$currqual;
+    }
+    $self->{'parsed'}->{'qualities'} = \@qualities;
 }
 
 =head2 _set_v3_peak_indices($buffer)
@@ -239,11 +240,11 @@ sub _set_v3_quality {
 =cut
 
 sub _set_v3_peak_indices {
-	my ($self,$buffer) = @_;
-	my $length = length($buffer);
-	my ($offset,@read,@positions);
-	@read = unpack "N$length",$buffer;
-     $self->{'parsed'}->{'peak_indices'} = join(' ',@read);
+    my ($self,$buffer) = @_;
+    my $length = length($buffer);
+    my ($offset,@read,@positions);
+    @read = unpack "N$length",$buffer;
+    $self->{'parsed'}->{'peak_indices'} = join(' ',@read);
 }
 
 =head2 _set_v3_base_accuracies($buffer)
@@ -259,19 +260,19 @@ sub _set_v3_peak_indices {
 
 #'
 sub _set_v3_base_accuracies {
-	my ($self,$buffer) = @_;
-	my $length = length($buffer);
-	my $qlength = $length/4;
-	my $offset = 0;
-	my (@qualities,@sorter,$counter,$round,$last_base);
-	foreach (qw(A C G T)) {
-		my @read;
-		$last_base = $offset + $qlength;
-		for (;$offset < $last_base; $offset += $qlength) {
-			@read = unpack "c$qlength", substr($buffer,$offset,$qlength);
-			$self->{'parsed'}->{'base_accuracies'}->{"$_"} = \@read;
-		}
+    my ($self,$buffer) = @_;
+    my $length = length($buffer);
+    my $qlength = $length/4;
+    my $offset = 0;
+    my (@qualities,@sorter,$counter,$round,$last_base);
+    foreach (qw(A C G T)) {
+	my @read;
+	$last_base = $offset + $qlength;
+	for (;$offset < $last_base; $offset += $qlength) {
+	    @read = unpack "c$qlength", substr($buffer,$offset,$qlength);
+	    $self->{'parsed'}->{'base_accuracies'}->{"$_"} = \@read;
 	}
+    }
 }
 
 
@@ -295,14 +296,14 @@ sub _set_comments {
     my $comments_retrieved = unpack "a$size",$buffer;
     $comments_retrieved =~ s/\0//;
     my @comments_split = split/\n/,$comments_retrieved;
-	if (@comments_split) {
-	    foreach (@comments_split) {
-		/(\w+)=(.*)/;
-		if ($1 && $2) {
-			$self->{'comments'}->{$1} = $2;
-		}
+    if (@comments_split) {
+	foreach (@comments_split) {
+	    /(\w+)=(.*)/;
+	    if ($1 && $2) {
+		$self->{'comments'}->{$1} = $2;
 	    }
 	}
+    }
     return;
 }
 
@@ -360,7 +361,7 @@ sub _set_v2_bases {
     my @read;
     for ($offset2=0;$offset2<$length;$offset2+=12) {
 	@read = unpack "N C C C C a C3", substr($buffer,$offset2,$length);
-     push @indices,$read[0];
+	push @indices,$read[0];
 	$currbase = uc($read[5]);
 	if ($currbase eq "A") { $currqual = $read[1]; }
 	elsif ($currbase eq "C") { $currqual = $read[2]; }
@@ -370,29 +371,13 @@ sub _set_v2_bases {
 	$sequence .= $currbase;
 	push @qualities,$currqual;
     }
-     unless (!@indices) {
-          $self->{'parsed'}->{'peak_indices'} = join(' ',@indices);
-     }
+    unless (!@indices) {
+	$self->{'parsed'}->{'peak_indices'} = join(' ',@indices);
+    }
     $self->{'parsed'}->{'sequence'} = $sequence;
-	unless (!@qualities) {
-		$self->{'parsed'}->{'qualities'} = join(' ',@qualities);
-	}
-}
-
-=head2 _initialize()
-
- Title   : _initialize()
- Usage   :
- Function: Bioperl initialize.
- Returns :
- Args    :
- Notes   :
-
-=cut
-
-sub _initialize {
-  my($self,@args) = @_;
-  return unless my $make = $self->SUPER::_initialize(@args);
+    unless (!@qualities) {
+	$self->{'parsed'}->{'qualities'} = join(' ',@qualities);
+    }
 }
 
 =head2 _set_v2_traces(\@traces_array)
@@ -407,23 +392,23 @@ sub _initialize {
 =cut
 
 sub _set_v2_traces {
-	my ($self,$rread) = @_;
-	my @read = @$rread;
-	my $array = 0;
-	for (my $offset2 = 0; $offset2< scalar(@read); $offset2+=4) {
-		if ($array) {
-			push @{$self->{'traces'}->{'A'}},$read[$offset2];
-			push @{$self->{'traces'}->{'C'}},$read[$offset2+1];
-			push @{$self->{'traces'}->{'G'}},$read[$offset2+3];
-			push @{$self->{'traces'}->{'T'}},$read[$offset2+2];
-		} else {
-			$self->{'traces'}->{'A'} .= " ".$read[$offset2];
-			$self->{'traces'}->{'C'} .= " ".$read[$offset2+1];
-			$self->{'traces'}->{'G'} .= " ".$read[$offset2+2];
-			$self->{'traces'}->{'T'} .= " ".$read[$offset2+3];
-		}
+    my ($self,$rread) = @_;
+    my @read = @$rread;
+    my $array = 0;
+    for (my $offset2 = 0; $offset2< scalar(@read); $offset2+=4) {
+	if ($array) {
+	    push @{$self->{'traces'}->{'A'}},$read[$offset2];
+	    push @{$self->{'traces'}->{'C'}},$read[$offset2+1];
+	    push @{$self->{'traces'}->{'G'}},$read[$offset2+3];
+	    push @{$self->{'traces'}->{'T'}},$read[$offset2+2];
+	} else {
+	    $self->{'traces'}->{'A'} .= " ".$read[$offset2];
+	    $self->{'traces'}->{'C'} .= " ".$read[$offset2+1];
+	    $self->{'traces'}->{'G'} .= " ".$read[$offset2+2];
+	    $self->{'traces'}->{'T'} .= " ".$read[$offset2+3];
 	}
-	return;
+    }
+    return;
 }
 
 =head2 get_trace($base_channel)
@@ -444,8 +429,8 @@ sub get_trace {
     if ($base_channel !~ /A|T|G|C/) {
 	$self->throw("You tried to ask for a base channel that wasn't A,T,G, or C. Ask for one of those next time.");
     } elsif ($base_channel) {
-     my @temp = split(' ',$self->{'traces'}->{$base_channel});
-     return \@temp;
+	my @temp = split(' ',$self->{'traces'}->{$base_channel});
+	return \@temp;
     }
 }
 
@@ -462,8 +447,8 @@ sub get_trace {
 
 sub get_peak_indices {
     my ($self) = shift;
-     my @temp = split(' ',$self->{'parsed'}->{'peak_indices'});
-     return \@temp;
+    my @temp = split(' ',$self->{'parsed'}->{'peak_indices'});
+    return \@temp;
 }
 
 
@@ -480,13 +465,13 @@ sub get_peak_indices {
 
 sub get_header {
     my ($self) = shift;
-     my %header;
-     foreach (qw(scf samples sample_offset bases bases_left_clip 
-          bases_right_clip bases_offset comment_size comments_offset 
-          version sample_size code_set peak_indices)) {
-          %header->{"$_"} = $self->{"$_"};
-     }
-     return \%header;
+    my %header;
+    foreach (qw(scf samples sample_offset bases bases_left_clip 
+		bases_right_clip bases_offset comment_size comments_offset 
+		version sample_size code_set peak_indices)) {
+	$header{"$_"} = $self->{"$_"};
+    }
+    return \%header;
 }
 
 =head2 _dump_traces_incoming($transformed)
@@ -506,20 +491,20 @@ sub get_header {
 
 #'
 sub _dump_traces_incoming {
-	my ($self) = @_;
-	my (@sA,@sT,@sG,@sC);
-		# @sA = @{$self->{'traces'}->{'A'}};
-		# @sC = @{$self->{'traces'}->{'C'}};
-		# @sG = @{$self->{'traces'}->{'G'}};
-		# @sT = @{$self->{'traces'}->{'T'}};
-		@sA = @{$self->get_trace('A')};
-		@sC = @{$self->get_trace('C')};
-		@sG = @{$self->get_trace('G')};
-		@sT = @{$self->get_trace('t')};
-	print ("Count\ta\tc\tg\tt\n");
-	for (my $curr=0; $curr < scalar(@sG); $curr++) {
-		print("$curr\t$sA[$curr]\t$sC[$curr]\t$sG[$curr]\t$sT[$curr]\n");
-	}
+    my ($self) = @_;
+    my (@sA,@sT,@sG,@sC);
+    # @sA = @{$self->{'traces'}->{'A'}};
+    # @sC = @{$self->{'traces'}->{'C'}};
+    # @sG = @{$self->{'traces'}->{'G'}};
+    # @sT = @{$self->{'traces'}->{'T'}};
+    @sA = @{$self->get_trace('A')};
+    @sC = @{$self->get_trace('C')};
+    @sG = @{$self->get_trace('G')};
+    @sT = @{$self->get_trace('t')};
+    print ("Count\ta\tc\tg\tt\n");
+    for (my $curr=0; $curr < scalar(@sG); $curr++) {
+	print("$curr\t$sA[$curr]\t$sC[$curr]\t$sG[$curr]\t$sT[$curr]\n");
+    }
     return;
 }
 
@@ -539,24 +524,24 @@ sub _dump_traces_incoming {
 =cut
 
 sub _dump_traces_outgoing {
-	my ($self,$transformed) = @_;
-	my (@sA,@sT,@sG,@sC);
-	if ($transformed) {
-		@sA = @{$self->{'text'}->{'t_samples_a'}};
-		@sC = @{$self->{'text'}->{'t_samples_c'}};
-		@sG = @{$self->{'text'}->{'t_samples_g'}};
-		@sT = @{$self->{'text'}->{'t_samples_t'}};
-	}
-	else {
-		@sA = @{$self->{'text'}->{'samples_a'}};
-		@sC = @{$self->{'text'}->{'samples_c'}};
-		@sG = @{$self->{'text'}->{'samples_g'}};
-		@sT = @{$self->{'text'}->{'samples_t'}};
-	}
-	print ("Count\ta\tc\tg\tt\n");
-	for (my $curr=0; $curr < scalar(@sG); $curr++) {
-		print("$curr\t$sA[$curr]\t$sC[$curr]\t$sG[$curr]\t$sT[$curr]\n");
-	}
+    my ($self,$transformed) = @_;
+    my (@sA,@sT,@sG,@sC);
+    if ($transformed) {
+	@sA = @{$self->{'text'}->{'t_samples_a'}};
+	@sC = @{$self->{'text'}->{'t_samples_c'}};
+	@sG = @{$self->{'text'}->{'t_samples_g'}};
+	@sT = @{$self->{'text'}->{'t_samples_t'}};
+    }
+    else {
+	@sA = @{$self->{'text'}->{'samples_a'}};
+	@sC = @{$self->{'text'}->{'samples_c'}};
+	@sG = @{$self->{'text'}->{'samples_g'}};
+	@sT = @{$self->{'text'}->{'samples_t'}};
+    }
+    print ("Count\ta\tc\tg\tt\n");
+    for (my $curr=0; $curr < scalar(@sG); $curr++) {
+	print("$curr\t$sA[$curr]\t$sC[$curr]\t$sG[$curr]\t$sT[$curr]\n");
+    }
     return;
 }
 
@@ -595,34 +580,34 @@ sub write_seq {
     unless (ref($swq) eq "Bio::Seq::SeqWithQuality") {
 	$self->throw("You must pass a Bio::Seq::SeqWithQuality object to write_seq as a parameter named \"SeqWithQuality\"");
     }
-	    # verify that there is some sequence or some qualities
-		# If the $swq with quality has no qualities, set all qualities to 0.
-		# If the $swq has no sequence, set the sequence to N\'s.
+    # verify that there is some sequence or some qualities
+    # If the $swq with quality has no qualities, set all qualities to 0.
+    # If the $swq has no sequence, set the sequence to N\'s.
     $self->_fill_missing_data($swq);
 
-	    # all of the rest of the arguments are comments for the scf
+    # all of the rest of the arguments are comments for the scf
     foreach $arg (sort keys %args) {
 	next if ($arg =~ /SeqWithQuality/i);
 	($label = $arg) =~ s/^\-//;
-	$comments{$label} = %args->{$arg};
+	$comments{$label} = $args{$arg};
     }
     if (!$comments{'NAME'}) { $comments{'NAME'} = $swq->id(); }
-	    # HA! Bwahahahaha.
+    # HA! Bwahahahaha.
     $comments{'CONV'} = "Bioperl-Chads Mighty SCF writer." unless defined $comments{'CONV'};
-		# now deal with the version of scf they want to write
-	if (%comments->{version}) {
-		if (%comments->{version} != 2 && %comments->{version} != 3) {
-			$self->warn("This module can only write version 2.0 or 3.0 scf's. Writing a version 2.0 scf by default.");
-			%comments->{version} = "2.00";
-		}
-		if (%comments->{'version'} > 2) {
-			%comments->{'version'} = "3.00";
-		}
+    # now deal with the version of scf they want to write
+    if ($comments{version}) {
+	if ($comments{version} != 2 && $comments{version} != 3) {
+	    $self->warn("This module can only write version 2.0 or 3.0 scf's. Writing a version 2.0 scf by default.");
+	    $comments{version} = "2.00";
 	}
-	else {
-		%comments->{'version'} = "2.00";
+	if ($comments{'version'} > 2) {
+	    $comments{'version'} = "3.00";
 	}
-	
+    }
+    else {
+	$comments{'version'} = "2.00";
+    }
+
 
 
     # set a few things in the header
@@ -631,89 +616,93 @@ sub write_seq {
     $self->{'header'}->{'bases'} = length($swq->seq());
     $self->{'header'}->{'bases_left_clip'} = "0";
     $self->{'header'}->{'bases_right_clip'} = "0";
-    $self->{'header'}->{'version'} = %comments->{'version'};
+    $self->{'header'}->{'version'} = $comments{'version'};
     $self->{'header'}->{'sample_size'} = "2";
     $self->{'header'}->{'code_set'} = "9";
     @{$self->{'header'}->{'spare'}} = qw(0 0 0 0 0 0 0 0 0 0 
 					 0 0 0 0 0 0 0 0 0 0);
 
-		# create the binary for the comments and file it in $self->{'binaries'}->{'comments'}
-	$self->_set_binary_comments(\%comments);
-		# create the binary and the strings for the traces, bases, offsets (if necessary), and accuracies (if necessary)
-	$self->_set_binary_tracesbases(%comments->{'version'},$swq->seq(),$swq->qual());
+    # create the binary for the comments and file it in $self->{'binaries'}->{'comments'}
+    $self->_set_binary_comments(\%comments);
+    # create the binary and the strings for the traces, bases, offsets (if necessary), and accuracies (if necessary)
+    $self->_set_binary_tracesbases($comments{'version'},$swq->seq(),$swq->qual());
 
-
-		# now set more things in the header
+    # now set more things in the header
     $self->{'header'}->{'samples_offset'} = "128";
 
-	my ($b_base_offsets,$b_base_accuracies,$samples_size,$bases_size);
-		#
-		# version 2
-		#
-	if ($self->{'header'}->{'version'} == 2) {
-	    $samples_size = $self->{'header'}->{'samples'} * 4 * 
-		               $self->{'header'}->{'sample_size'};
-	   $bases_size = length($swq->seq()) * 12;
-	    $self->{'header'}->{'bases_offset'} = 128 + length($self->{'binaries'}->{'samples_all'});
-	    $self->{'header'}->{'comments_offset'} = 128 + length($self->{'binaries'}->{'samples_all'}) + length($self->{'binaries'}->{'v2_bases'}); 
-	    $self->{'header'}->{'comments_size'} = length($self->{'binaries'}->{'comments'});
-	    $self->{'header'}->{'private_size'} = "0";
-	    $self->{'header'}->{'private_offset'} = 128 + $samples_size + 
-		$bases_size + $self->{'header'}->{'comments_size'};
-	}
-	else {
-		$self->{'header'}->{'bases_offset'} = 128 + length($self->{'binaries'}->{'samples_all'});
-		$self->{'header'}->{'comments_size'} = length($self->{'binaries'}->{'comments'});
-			# this is:
-			# bases_offset + base_offsets + accuracies + called_bases + reserved
-		$self->{'header'}->{'comments_offset'} = $self->{'header'}->{'bases_offset'} + 4*$self->{header}->{'bases'} + 4*$self->{header}->{'bases'} + $self->{header}->{'bases'} + 3*$self->{header}->{'bases'}; 
-		$self->{'header'}->{'private_size'} = "0";
-		$self->{'header'}->{'private_offset'} = $self->{'header'}->{'comments_offset'} + $self->{'header'}->{'comments_size'};
-	}
+    my ($b_base_offsets,$b_base_accuracies,$samples_size,$bases_size);
+    #
+    # version 2
+    #
+    if ($self->{'header'}->{'version'} == 2) {
+	$samples_size = $self->{'header'}->{'samples'} * 4 * 
+	    $self->{'header'}->{'sample_size'};
+	$bases_size = length($swq->seq()) * 12;
+	$self->{'header'}->{'bases_offset'} = 128 + length($self->{'binaries'}->{'samples_all'});
+	$self->{'header'}->{'comments_offset'} = 128 + length($self->{'binaries'}->{'samples_all'}) + length($self->{'binaries'}->{'v2_bases'}); 
+	$self->{'header'}->{'comments_size'} = length($self->{'binaries'}->{'comments'});
+	$self->{'header'}->{'private_size'} = "0";
+	$self->{'header'}->{'private_offset'} = 128 + $samples_size + 
+	    $bases_size + $self->{'header'}->{'comments_size'};
+    }
+    else {
+	$self->{'header'}->{'bases_offset'} = 128 + length($self->{'binaries'}->{'samples_all'});
+	$self->{'header'}->{'comments_size'} = length($self->{'binaries'}->{'comments'});
+	# this is:
+	# bases_offset + base_offsets + accuracies + called_bases + reserved
+	$self->{'header'}->{'comments_offset'} = $self->{'header'}->{'bases_offset'} + 4*$self->{header}->{'bases'} + 4*$self->{header}->{'bases'} + $self->{header}->{'bases'} + 3*$self->{header}->{'bases'}; 
+	$self->{'header'}->{'private_size'} = "0";
+	$self->{'header'}->{'private_offset'} = $self->{'header'}->{'comments_offset'} + $self->{'header'}->{'comments_size'};
+    }
 
-	$self->_set_binary_header();
+    $self->_set_binary_header();
 
     # should something better be done rather then returning after
     # writing? I don't do any exception trapping here
-	if (%comments->{'version'} == 2) {
-		    	# print ("Lengths:\n");
-			# print("Header  : ".length($self->{'binaries'}->{'header'})."\n");
-			# print("Traces  : ".length($self->{'binaries'}->{'samples_all'})."\n");
-			# print("Bases   : ".length($self->{'binaries'}->{'v2_bases'})."\n");
-			# print("Comments: ".length($self->{'binaries'}->{'comments'})."\n");
-	    $self->_print ($self->{'binaries'}->{'header'}) or return;
-	    $self->_print ($self->{'binaries'}->{'samples_all'}) or return;
-	    $self->_print ($self->{'binaries'}->{'v2_bases'}) or return;
-	    $self->_print ($self->{'binaries'}->{'comments'}) or return;
-	}
-	elsif (%comments->{'version'} ==3) {
-			# print ("Lengths:\n");
-			# print("Header  : ".length($self->{'binaries'}->{'header'})."\n");
-			# print("Traces  : ".length($self->{'binaries'}->{'samples_all'})."\n");
-			# print("Offsets : ".length($self->{'binaries'}->{'v3_peak_offsets'})."\n");
-			# print("Accuracy: ".length($self->{'binaries'}->{'v3_accuracies_all'})."\n");
-			# print("Bases   : ".length($self->{'binaries'}->{'v3_called_bases'})."\n");
-			# print("Reserved: ".length($self->{'binaries'}->{'v3_reserved'})."\n");
-			# print("Comments: ".length($self->{'binaries'}->{'comments'})."\n");
-		$self->{'header'}->{'comments_offset'} = 
-			128+length($self->{'binaries'}->{'samples_all'})+
-			length($self->{'binaries'}->{'v3_peak_offsets'})+
-			length($self->{'binaries'}->{'v3_accuracies_all'})+
+    if ($comments{'version'} == 2) {
+	# print ("Lengths:\n");
+	# print("Header  : ".length($self->{'binaries'}->{'header'})."\n");
+	# print("Traces  : ".length($self->{'binaries'}->{'samples_all'})."\n");
+	# print("Bases   : ".length($self->{'binaries'}->{'v2_bases'})."\n");
+	# print("Comments: ".length($self->{'binaries'}->{'comments'})."\n");
+	$self->_print ($self->{'binaries'}->{'header'}) or return;
+	$self->_print ($self->{'binaries'}->{'samples_all'}) or return;
+	$self->_print ($self->{'binaries'}->{'v2_bases'}) or return;
+	$self->_print ($self->{'binaries'}->{'comments'}) or return;
+    }
+    elsif ($comments{'version'} ==3) {
+	# print ("Lengths:\n");
+	# print("Header  : ".length($self->{'binaries'}->{'header'})."\n");
+	# print("Traces  : ".length($self->{'binaries'}->{'samples_all'})."\n");
+	# print("Offsets : ".length($self->{'binaries'}->{'v3_peak_offsets'})."\n");
+	# print("Accuracy: ".length($self->{'binaries'}->{'v3_accuracies_all'})."\n");
+	# print("Bases   : ".length($self->{'binaries'}->{'v3_called_bases'})."\n");
+	# print("Reserved: ".length($self->{'binaries'}->{'v3_reserved'})."\n");
+	# print("Comments: ".length($self->{'binaries'}->{'comments'})."\n");
+	$self->{'header'}->{'comments_offset'} = 
+	    128+length($self->{'binaries'}->{'samples_all'})+
+		length($self->{'binaries'}->{'v3_peak_offsets'})+
+		    length($self->{'binaries'}->{'v3_accuracies_all'})+
 			length($self->{'binaries'}->{'v3_called_bases'})+
-			length($self->{'binaries'}->{'v3_reserved'});
-		$self->{'header'}->{'spare'}->[1] = 
-			$self->{'header'}->{'comments_offset'} +
-			length($self->{'binaries'}->{'comments'});
-		$self->_set_binary_header();
-	    $self->_print ($self->{'binaries'}->{'header'}) or print("Couldn't write header\n");
-	    $self->_print ($self->{'binaries'}->{'samples_all'}) or print("Couldn't write samples\n");
-	    $self->_print ($self->{'binaries'}->{'v3_peak_offsets'}) or print("Couldn't write peak offsets\n");
-	    $self->_print ($self->{'binaries'}->{'v3_accuracies_all'}) or print("Couldn't write accuracies\n");
-	    $self->_print ($self->{'binaries'}->{'v3_called_bases'}) or print("Couldn't write called_bases\n");
-	    $self->_print ($self->{'binaries'}->{'v3_reserved'}) or print("Couldn't write reserved\n");
-	    $self->_print ($self->{'binaries'}->{'comments'}) or print ("Couldn't write comments\n");
-	}
-	$self->close();
+			    length($self->{'binaries'}->{'v3_reserved'});
+	$self->{'header'}->{'spare'}->[1] = 
+	    $self->{'header'}->{'comments_offset'} +
+		length($self->{'binaries'}->{'comments'});
+	$self->_set_binary_header();
+	$self->_print ($self->{'binaries'}->{'header'}) or print("Couldn't write header\n");
+	$self->_print ($self->{'binaries'}->{'samples_all'}) or print("Couldn't write samples\n");
+	$self->_print ($self->{'binaries'}->{'v3_peak_offsets'}) or print("Couldn't write peak offsets\n");
+	$self->_print ($self->{'binaries'}->{'v3_accuracies_all'}) or print("Couldn't write accuracies\n");
+	$self->_print ($self->{'binaries'}->{'v3_called_bases'}) or print("Couldn't write called_bases\n");
+	$self->_print ($self->{'binaries'}->{'v3_reserved'}) or print("Couldn't write reserved\n");
+	$self->_print ($self->{'binaries'}->{'comments'}) or print ("Couldn't write comments\n");
+    }
+
+    # kinda unnecessary, given the close() below, but maybe that'll go
+    # away someday.
+    $self->flush if $self->_flush_on_write && defined $self->_fh;
+
+    $self->close();
 }
 
 =head2 _set_binary_header()
@@ -1247,6 +1236,25 @@ sub _dump_base_accuracies_incoming {
 }
 
 
+=head2 _dump_comments()
+
+ Title   : _dump_comments()
+ Usage   : $self->_dump_comments();
+ Function: Debug dump the comments section from the scf.
+ Returns : Nothing.
+ Args    : Nothing.
+ Notes   : None.
+
+=cut
+
+sub _dump_comments {
+    my ($self) = @_;
+    warn ("SCF comments:\n");
+    foreach my $k (keys %{$self->{'comments'}}) {
+	warn ("\t {$k} ==> ", $self->{'comments'}->{$k}, "\n");
+    }
+}
+
 
 1;
 __END__
