@@ -1,4 +1,4 @@
-
+# $Id$
 #
 # BioPerl module for Bio::Tools::AlignFactory
 #
@@ -29,48 +29,40 @@ http://bio.perl.org/ or birney@sanger.ac.uk
 
 =head1 APPENDIX
 
-The rest of the documentation details each of the object methods. Internal methods are usually preceded with a _
+The rest of the documentation details each of the object
+methods. Internal methods are usually preceded with a _
 
 =cut
 
 
 # Let the code begin...
 
-
 package Bio::Tools::AlignFactory;
-use vars qw($AUTOLOAD @ISA);
+use vars qw(@ISA);
 use strict;
 
-# Object preamble - inheriets from Bio::Root::Object
-
-use Bio::Root::Object;
+use Bio::Root::Root;
+@ISA = qw(Bio::Root::Root);
 
 BEGIN {
     eval {
-	require bp_sw;
+	require Bio::Ext::Align;
     };
     if ( $@ ) {
-	print STDERR ("\nThe C-compiled engine for Smith Waterman alignments (bp_sw) has not been installed.\n Please read the installation instructions for bioperl for using the compiled extensions\n\n");
+	print STDERR ("\nThe C-compiled engine for Smith Waterman alignments (Bio::Ext::Align) has not been installed.\n Please install the bioperl-ext package\n\n");
 	exit(1);
     }
 }
 
-
-@ISA = qw(Bio::Root::Object Exporter);
-# new() is inherited from Bio::Root::Object
-
-# _initialize is where the heavy stuff will happen when new is called
-
-sub _initialize {
-  my($self,@p) = @_;
-  my $make = $self->SUPER::_initialize(@p);
-  
+sub new {
+  my($class,@args) = @_;
+  my $self = $class->SUPER::new(@args);
+  $self->_initialize(@args);
   # set up defaults
-
+  
   $self->{'kbyte'} = 20000;
-  $self->{'report'} = 0;
-# set stuff in self from @args
- return $make; # success - we hope!
+  $self->{'report'} = 0;  
+  return $self;
 }
 
 
@@ -92,7 +84,6 @@ sub kbyte {
     if( defined $value ) {
 	$self->{'kbyte'} = $value;
     } 
-
     return $self->{'kbyte'};
 }
 
@@ -142,14 +133,13 @@ sub set_memory_and_report{
        $self->throw("You can suggest aligning things with less than 5kb");
    }
 
-   &bp_sw::change_max_BaseMatrix_kbytes($self->{'kbyte'});
+   &Bio::Ext::Align::change_max_BaseMatrix_kbytes($self->{'kbyte'});
 
    if( $self->{'report'} == 0 ) {
-       &bp_sw::error_off(16);
+       &Bio::Ext::Align::error_off(16);
    } else {
-       &bp_sw::error_on(16);
+       &Bio::Ext::Align::error_on(16);
    }
 }
 
-
-
+1;
