@@ -1,4 +1,4 @@
-# $Id: BPlite.pm,v 1.32 2002/01/04 14:50:27 jason Exp $
+# $Id$
 ##############################################################################
 # Bioperl module Bio::Tools::BPlite
 ##############################################################################
@@ -27,6 +27,7 @@ Bio::Tools::BPlite - Lightweight BLAST parser
 	    $hsp->bits;
 	    $hsp->percent;
 	    $hsp->P;
+            $hsp->EXP;
 	    $hsp->match;
 	    $hsp->positive;
 	    $hsp->length;
@@ -37,7 +38,7 @@ Bio::Tools::BPlite - Lightweight BLAST parser
 	    $hsp->query->end;
 	    $hsp->hit->start;
 	    $hsp->hit->end;
-	    $hsp->hit->seqname;
+	    $hsp->hit->seq_id;
 	    $hsp->hit->overlaps($exon);
 	}
     }
@@ -93,11 +94,11 @@ have one attribute (name) and one method (nextHSP).
      # canonical form is again a while loop
  }
 
-An HSP is a high scoring pair, or simply an alignment. 
-HSP objects inherit all the useful methods from RangeI/SeqFeatureI/FeaturePair,
-but provide an additional set of attributes (score, bits, percent, P, match, 
-positive, length, querySeq, sbjctSeq, homologySeq) that should be familiar to
-anyone who has seen a blast report. 
+An HSP is a high scoring pair, or simply an alignment.  HSP objects
+inherit all the useful methods from RangeI/SeqFeatureI/FeaturePair,
+but provide an additional set of attributes (score, bits, percent, P,
+match, EXP, positive, length, querySeq, sbjctSeq, homologySeq) that
+should be familiar to anyone who has seen a blast report.
 
 For lazy/efficient coders, two-letter abbreviations are available for the 
 attributes with long names (qs, ss, hs). Ranges of the aligned sequences in
@@ -118,7 +119,7 @@ contain the alignment sequences from the blast report.
  $hsp->homologySeq;   $hsp->hs;
  $hsp->query->start;
  $hsp->query->end;
- $hsp->query->seqname;
+ $hsp->query->seq_id;
  $hsp->hit->primary_tag; # "similarity"
  $hsp->hit->source_tag;  # "BLAST"
  $hsp->hit->start;
@@ -157,7 +158,7 @@ Univeristy, St. Louis, MO.
 
 =head1 CONTRIBUTORS
 
-Jason Stajich, jason@cgt.mc.duke.edu
+Jason Stajich, jason@bioperl.org
 
 =head1 COPYRIGHT
 
@@ -327,7 +328,7 @@ sub nextSbjct {
     if    ($_ !~ /\w/)            {next}
     elsif ($_ =~ /Strand HSP/)    {next} # WU-BLAST non-data
     elsif ($_ =~ /^\s{0,2}Score/) {$self->_pushback($_); last}
-    elsif ($_ =~ /^Parameters|^\s+Database:|^\s+Posted date:/) {
+    elsif ($_ =~ /^Histogram|^Searching|^Parameters|^\s+Database:|^\s+Posted date:/) {
 	$self->_pushback($_); 
 	last;
     }
@@ -372,7 +373,14 @@ sub _parseHeader {
 	  $header_flag = 1;	# valid header element found
 	  my $query = $1;
 	  while( defined($_ = $self->_readline() ) ) {
-	      if( $_ =~ /^Database/ ) { $self->_pushback($_); last; }
+	      # Continue reading query name until encountering either
+	      # a line that starts with "Database" or a blank line.
+	      # The latter condition is needed in order to be able to
+	      # parse megablast output correctly, since Database comes
+	      # before (not after) the query.
+	      if( ($_ =~ /^Database/) || ($_ =~ /^$/) ) {
+		  $self->_pushback($_); last;
+	      }	      
 	      $query .= $_;
 	  }
 	  $query =~ s/\s+/ /g;
@@ -392,6 +400,12 @@ sub _parseHeader {
 	  $self->{'BLAST_TYPE'} = $2; 
 	  $self->{'BLAST_VERSION'} = $3;
       }				# BLAST report type - not a valid header element # JB949
+      
+      # Support Paracel BTK output
+      elsif ( $_ =~ /(^[A-Z0-9_]+)\s+BTK\s+/ ) { 
+	  $self->{'BLAST_TYPE'} = $1;
+	  $self->{'BTK'} = 1;
+     } 
       elsif ($_ =~ /^Database:\s+(.+)/) {$header_flag = 1;$self->{'DATABASE'} = $1} # valid header element found
       elsif ($_ =~ /^\s*pattern\s+(\S+).*position\s+(\d+)\D/) {   
 	  # For PHIBLAST reports
@@ -404,6 +418,12 @@ sub _parseHeader {
 	  $self->_pushback($_);
 	  return 0;		# there's nothing in the report
       }
+      # bug fix suggested by MI Sadowski via Martin Lomas
+      # see bug report #1118
+      if( ref($self->_fh()) !~ /GLOB/ && $self->_fh()->can('EOF') && eof($self->_fh()) ) {
+	  $self->warn("unexpected EOF in file\n");
+	  return -1;
+      }
   }
   return -1; # EOF
 }
@@ -412,14 +432,16 @@ sub _fastForward {
     my ($self) = @_;
     return 0 if $self->{'REPORT_DONE'}; # empty report
     while(defined( $_ = $self->_readline() ) ) {
-	if ($_ =~ /^Parameters|^\s+Database:|^\s+Posted date:/) {
+	if ($_ =~ /^Histogram|^Searching|^Parameters|^\s+Database:|^\s+Posted date:/) {
 	    return 0;
 	} elsif( $_ =~ /^>/ ) {
 	    $self->_pushback($_);	
 	    return 1;
 	}
     }
-    $self->warn("Possible error (1) while parsing BLAST report!");
+    unless( $self->{'BTK'} ) { # Paracel BTK reports have no footer
+	$self->warn("Possible error (1) while parsing BLAST report!");
+    }
 }
 
 1;
