@@ -1,4 +1,4 @@
-# $Id: Results.pm,v 1.17.2.3 2002/07/04 17:04:07 jason Exp $
+# $Id$
 #
 # Perl Module for HMMResults
 #
@@ -29,7 +29,7 @@ Bio::Tools::HMMER::Results - Object representing HMMER output results
 
    # alternative way of getting out all domains directly
    foreach $domain ( $res->each_Domain ) {
-       print "Domain on ",$domain->seqname," with score ",
+       print "Domain on ",$domain->seq_id," with score ",
        $domain->bits," evalue ",$domain->evalue,"\n";
    }
 
@@ -64,7 +64,7 @@ the bugs and their resolution.  Bug reports can be submitted via email
 or the web:
 
   bioperl-bugs@bio.perl.org
-  http://www.bioperl.org/bioperl-bugs/
+  http://www.bugzilla.bioperl.org/
 
 =head1 AUTHOR - Ewan Birney
 
@@ -236,7 +236,7 @@ sub add_Domain {
     my $unit = shift;
     my $name;
 
-    $name = $unit->seqname();
+    $name = $unit->seq_id();
 
     if( ! exists $self->{'seq'}->{$name} ) {
 	$self->warn("Adding a domain of $name but with no HMMSequence. Will be kept in domain array but not added to a HMMSequence");
@@ -295,7 +295,9 @@ sub domain_bits_cutoff_from_evalue {
     @doms = $self->each_Domain;
 
 
-    @doms = sort { $b->bits <=> $a->bits } @doms;
+    @doms = map { $_->[0] } 
+            sort { $b->[1] <=> $a->[1] } 
+            map { [ $_, $_->bits] } @doms;
     $seen = 0;
     foreach $_ ( @doms ) {
 	if( $_->evalue > $eval ) {
@@ -360,7 +362,8 @@ sub write_FT_output {
     foreach $seq ( $self->each_Set() ) {
 	print $file sprintf("ID   %s\n",$seq->name());
 	foreach $unit ( $seq->each_Domain() ) {
-	    print $file sprintf("FT   %s   %d %d %s\n",$idt,$unit->start,$unit->end,$unit->hmmname);
+	    print $file sprintf("FT   %s   %d %d %s\n",$idt,
+				$unit->start,$unit->end,$unit->hmmname);
 	}
 	print $file "//\n";
     }
@@ -393,7 +396,6 @@ sub filter_on_cutoff {
     foreach $seq ( $self->each_Set()) {
 	next if( $seq->bits() < $seqthr );
 	$new->add_Set($seq);
-	
 	foreach $unit ( $seq->each_Domain() ) {
 	    next if( $unit->bits() < $domthr );
 	    $new->add_Domain($unit);
@@ -429,7 +431,10 @@ sub write_ascii_out {
 
     foreach $seq ( $self->each_Set()) {
 	foreach $unit ( $seq->each_Domain()) {
-	    print $fh sprintf("%s %4d %4d %s %4d %4d %4.2f %4.2g %s\n",$unit->seqname(),$unit->start(),$unit->end(),$unit->hmmacc,$unit->hstart,$unit->hend,$unit->bits,$unit->evalue,$unit->hmmname);
+	    print $fh sprintf("%s %4d %4d %s %4d %4d %4.2f %4.2g %s\n",
+			      $unit->seq_id(),$unit->start(),$unit->end(),
+			      $unit->hmmacc,$unit->hstart,$unit->hend,
+			      $unit->bits,$unit->evalue,$unit->hmmname);
 	}
     }
 	    
@@ -475,8 +480,8 @@ sub write_GDF_bits {
     }
 
     @narray = sort { my ($aa,$bb,$st_a,$st_b); 
-		     $aa = $a->seqname(); 
-		     $bb = $b->seqname(); 
+		     $aa = $a->seq_id(); 
+		     $bb = $b->seq_id(); 
 		     if ( $aa eq $bb) {
 			 $st_a = $a->start();
 			 $st_b = $b->start();
@@ -487,7 +492,7 @@ sub write_GDF_bits {
 		     } } @array;
 
     foreach $unit ( @narray ) {
-	print $file sprintf("%-24s\t%6d\t%6d\t%15s\t%.1f\t%g\n",$unit->get_nse(),$unit->start(),$unit->end(),$unit->seqname(),$unit->bits(),$unit->evalue);
+	print $file sprintf("%-24s\t%6d\t%6d\t%15s\t%.1f\t%g\n",$unit->get_nse(),$unit->start(),$unit->end(),$unit->seq_id(),$unit->bits(),$unit->evalue);
     }
 
 }
@@ -544,7 +549,7 @@ sub write_GDF {
 
 
     foreach $unit ( $self->eachHMMUnit() ) {
-	print $file sprintf("%-24s\t%6d\t%6d\t%15s\t%.1f\t%g\n",$unit->get_nse(),$unit->start(),$unit->end(),$unit->seqname(),$unit->bits(),$unit->evalue);
+	print $file sprintf("%-24s\t%6d\t%6d\t%15s\t%.1f\t%g\n",$unit->get_nse(),$unit->start(),$unit->end(),$unit->seq_id(),$unit->bits(),$unit->evalue);
     }
     
 }
@@ -715,6 +720,7 @@ sub _parse_hmmpfam {
 	    %hash = ();
 	    
 	    while(<$file>){
+
 		if( /Accession:\s+(\S+)/ ) { $seq->accession($1); next }
 		elsif( s/^Description:\s+// ) { chomp; $seq->desc($_); next } 
 		/^Parsed for domains/ && last;
@@ -722,7 +728,7 @@ sub _parse_hmmpfam {
 		# This is to parse out the accession numbers in old Pfam format.
 		# now not support due to changes in HMMER.
 
-		if( (($id,$acc, $sc, $ev, $nd) = /^\s*(\S+)\s+([A-Za-z]+\d+).+?\s(\S+)\s+(\S+)\s+(\d+)\s*$/)) {
+		if( (($id,$acc, $sc, $ev, $nd) = /^\s*(\S+)\s+(\S+).+?\s(\S+)\s+(\S+)\s+(\d+)\s*$/)) {
 		    $hash{$id} = $sc; # we need this for the sequence 
 		                      # core of the domains below!
 		    $acc {$id} = $acc;
@@ -749,7 +755,7 @@ sub _parse_hmmpfam {
 		if( (($id, $sqfrom, $sqto, $hmmf,$hmmt,$sc, $ev) = 
 		     /(\S+)\s+\S+\s+(\d+)\s+(\d+).+?(\d+)\s+(\d+)\s+\S+\s+(\S+)\s+(\S+)\s*$/)) {
 		    $unit = Bio::Tools::HMMER::Domain->new();
-		    $unit->seqname  ($seqname);
+		    $unit->seq_id  ($seqname);
 		    $unit->hmmname  ($id);
 		    $unit->start    ($sqfrom);
 		    $unit->end      ($sqto);
@@ -764,7 +770,7 @@ sub _parse_hmmpfam {
 
 		    $unit->seqbits($hash{$id});
 	    
-		    if( exists $acc{$id} ) {
+		    if( defined $acc{$id} ) {
 			$unit->hmmacc($acc{$id});
 		    }
 
@@ -853,11 +859,14 @@ sub get_unit_nse {
 sub _parse_hmmsearch {
     my $self = shift;
     my $file = shift;
-    my ($id,$sqfrom,$sqto,$sc,$ev,$unit,$nd,$seq,$hmmf,$hmmt,$hmmfname,%seqh);
+    my ($id,$sqfrom,$sqto,$sc,$ev,$unit,$nd,$seq,$hmmf,$hmmt,
+	$hmmfname,$hmmacc, $hmmid, %seqh);
     my $count = 0;
     
     while(<$file>) {
         /^HMM file:\s+(\S+)/ and do { $self->hmmfile($1); $hmmfname = $1 };
+	/^Accession:\s+(\S+)/ and do { $hmmacc = $1 };
+	/^Query HMM:\s+(\S+)/ and do { $hmmid = $1 };	
 	/^Sequence database:\s+(\S+)/ and do { $self->seqfile($1) };
         /^Scores for complete sequences/ && last;
     }
@@ -873,6 +882,7 @@ sub _parse_hmmsearch {
 	    $seqh{$id} = $sc;
 	    $seq->evalue($ev);
 	    $self->add_Set($seq);
+	    $seq->accession($hmmacc);
 	}
     }
 
@@ -881,7 +891,7 @@ sub _parse_hmmsearch {
 	if( (($id, $sqfrom, $sqto, $hmmf, $hmmt, $sc, $ev) = /(\S+)\s+\S+\s+(\d+)\s+(\d+).+?(\d+)\s+(\d+)\s+\S+\s+(\S+)\s+(\S+)\s*$/)) {
 	    $unit = Bio::Tools::HMMER::Domain->new();
 
-	    $unit->seqname($id);
+	    $unit->seq_id($id);
 	    $unit->hmmname($hmmfname);
 	    $unit->start($sqfrom);
 	    $unit->end($sqto);
