@@ -1,4 +1,4 @@
-# $Id: StandAloneBlast.pm,v 1.18.2.6 2002/03/16 18:55:52 peter Exp $
+# $Id$
 #
 # BioPerl module for Bio::Tools::StandAloneBlast
 #
@@ -48,12 +48,11 @@ DESCRIPTION section for details.
 
 =head1 DESCRIPTION
 
-This DESCRIPTION only documents Bio::Tools::Run::StandAloneBlast.pm: - a
+This DESCRIPTION only documents Bio::Tools::Run::StandAloneBlast: - a
 Bioperl object for running the NCBI standAlone BLAST package.  Blast,
 itself, is a large & complex program - for more information regarding
 BLAST, please see the BLAST documentation which accompanies the BLAST
-distribution. BLAST is available from:
-ftp://ftp.ncbi.nlm.nih.gov/blast/server/current_release/.
+distribution. BLAST is available from ftp://ncbi.nlm.nih.gov/blast/.
 
 (A source of confusion in documenting a BLAST interface is that the
 term "program" is used in - at least - three different ways in the
@@ -67,22 +66,22 @@ implemented by using specific combinations of BLAST executables,
 programs and parameters.  They will be referred by their specific
 names - eg PSIBLAST and PHIBLAST. )
 
-StandAloneBlast.pm has been tested so far only under Linux. I expect
+StandAloneBlast has been tested so far only under Linux. I expect
 that it should also work under other Unix systems. However, since the
 module is implemented using (unix) system calls, modification may be
-necessary before StandAloneBlast.pm would work under non-Unix
+necessary before StandAloneBlast would work under non-Unix
 operating systems (eg Windows, MacOS).  Before running
-StandAloneBlast.pm it is necessary: to install BLAST on your system,
+StandAloneBlast it is necessary: to install BLAST on your system,
 to edit set the environmental variable $BLASTDIR or your $PATH
 variable to point to the BLAST directory, and to ensure that users
 have execute privileges for the BLAST program.  If the databases
 which will be searched by BLAST are located in the data subdirectory
 of the blast program directory (the default installation location),
-StandAloneBlast.pm will find them; however, if the database files are
+StandAloneBlast will find them; however, if the database files are
 located in any other location, environmental variable $BLASTDATADIR
 will need to be set to point to that directory.
 
-The use of the StandAloneBlast.pm module is as follows: Initially, a
+The use of the StandAloneBlast module is as follows: Initially, a
 local blast "factory object" is created. The constructor may be passed
 an optional array of (non-default) parameters to be used by the
 factory, eg:
@@ -135,28 +134,16 @@ In addition, sequence input may be in the form of either a Bio::Seq
  $blast_report = $factory->blastall($input);
 
 For blastall and non-psiblast blastpgp runs, report object is either a
-BPlite.pm or Blast.pm object, selected by the user with the parameter
-_READMETHOD.  (The leading underscore is needed to distinguish this
-option from options which are passed to the BLAST executable.) The
-default parser is BPlite.  For (multiple iteration) psiblast and
-bl2seq runs the report is automatically parsed by the BPpsilite.pm and
-BPbl2seq.pm parsers respectively, since neither Blast.pm nor BPlite
-can parse these reports. In any case, the "raw" blast report is also
-available. The filename is set by the in the 'outfile' parameter and
-has the default value of "blastreport.out".
-
-When using the Blast.pm parser, only a default configuration is currently supported:
-
-        -signif => $self->e()  || 1e-5, # where $self->e(), if set, is the BLAST cutoff value
-	-parse  => 1,
-	-stats  => 1,
-	-check_all_hits => 1,
-
-If it is desired to parse the resulting report with Blast.pm with
-other values, the user can save the report in the file given by
- $factory-E<gt>outfile('outputfilelocation') 
-and then reading that file with Blast.pm using any parameters desired.
-
+BPlite.pm or Bio::SearchIO object, selected by the user with the
+parameter _READMETHOD.  (The leading underscore is needed to
+distinguish this option from options which are passed to the BLAST
+executable.) The default parser is Bio::SearchIO::blast.  For
+(multiple iteration) psiblast and bl2seq runs the report is
+automatically parsed by the BPpsilite.pm and BPbl2seq.pm parsers
+respectively, since neither Blast.pm nor BPlite can parse these
+reports. In any case, the "raw" blast report is also available. The
+filename is set by the in the 'outfile' parameter and has the default
+value of "blastreport.out".
 
 For psiblast execution in BLAST's "jumpstart" mode, the program must
 be passed (in addition to the query sequence itself) an alignment
@@ -196,9 +183,10 @@ For more examples of syntax and use of Blast.pm, the user is
 encouraged to run the scripts standaloneblast.pl in the bioperl
 /examples directory and StandAloneBlast.t in the bioperl /t directory.
 
-Note: There is a similar (but older) perl object interface offered by nhgri. The nhgri module
-only supports blastall and does not support blastpgp, psiblast, phiblast, bl2seq etc.
-This module can be found at http://genome.nhgri.nih.gov/blastall/.
+Note: There is a similar (but older) perl object interface offered by
+nhgri. The nhgri module only supports blastall and does not support
+blastpgp, psiblast, phiblast, bl2seq etc.  This module can be found at
+http://genome.nhgri.nih.gov/blastall/.
 
 =head1 DEVELOPERS NOTES
 
@@ -240,8 +228,9 @@ methods. Internal methods are usually preceded with a _
 
 package Bio::Tools::Run::StandAloneBlast;
 
-use vars qw($AUTOLOAD @ISA %PROGRAMS @BLASTALL_PARAMS @BLASTPGP_PARAMS 
-	    @BL2SEQ_PARAMS @OTHER_PARAMS %OK_FIELD $DATADIR $BLASTDIR 
+use vars qw($AUTOLOAD @ISA $PROGRAMDIR  $DATADIR 
+	    @BLASTALL_PARAMS @BLASTPGP_PARAMS 
+	    @BL2SEQ_PARAMS @OTHER_PARAMS %OK_FIELD
 	    );
 use strict;
 use Bio::Root::Root;
@@ -251,6 +240,8 @@ use Bio::SeqIO;
 use Bio::Tools::BPbl2seq;
 use Bio::Tools::BPpsilite;
 use Bio::SearchIO;
+use Bio::Tools::Run::WrapperBase;
+use Bio::Factory::ApplicationFactoryI;
 
 BEGIN {      
 
@@ -281,14 +272,16 @@ BEGIN {
 #  2. include a definition of an environmental variable BLASTDIR in every script that will
 #     use StandAloneBlast.pm.
 #	BEGIN {$ENV{BLASTDIR} = '/home/peter/blast/'; }
-     $BLASTDIR = $ENV{'BLASTDIR'} || '';
-
+     $PROGRAMDIR = $ENV{'BLASTDIR'} || '';
+     
 # If local BLAST databases are not stored in the standard
 # /data directory, the variable BLASTDATADIR will need to be set explicitly 
      $DATADIR =  $ENV{'BLASTDATADIR'} || $ENV{'BLASTDB'} || '';
 }
 
-@ISA = qw(Bio::Root::Root Bio::Root::IO);
+@ISA = qw(Bio::Root::Root 
+	  Bio::Tools::Run::WrapperBase 
+	  Bio::Factory::ApplicationFactoryI);
 
 =head1 BLAST parameters
 
@@ -342,20 +335,18 @@ sub new {
     my ($caller, @args) = @_;
     # chained new
     my $self = $caller->SUPER::new(@args);
+ 
     # to facilitiate tempfile cleanup
-    $self->_initialize_io();
-    unless (&Bio::Tools::Run::StandAloneBlast::exists_blast()) {
-	$self->debug( "Blast program not found or not executable. \n  Blast can be obtained from ftp://ftp.ncbi.nlm.nih.gov/blast/server/current_release/\n");    
-    } 
-    # to facilitiate tempfile cleanup
-    $self->_initialize_io();
-    my ($fh,$tempfile) = $self->tempfile();
+    my ($tfh,$tempfile) = $self->io->tempfile();
+    close($tfh); # we don't want the filehandle, just a temporary name
     $self->outfile($tempfile);
-    $self->_READMETHOD('BPlite');
+    $self->_READMETHOD('Blast');
     while (@args)  {
 	my $attr =   shift @args;
 	my $value =  shift @args;
 	next if( $attr eq '-verbose');
+	# the workaround to deal with initializing
+	$attr = 'p' if $attr =~ /^\s*program\s*$/;
 	$self->$attr($value);
     }
     return $self;
@@ -370,53 +361,95 @@ sub AUTOLOAD {
     # actual key is first letter of $attr unless first attribute
     # letter is underscore (as in _READMETHOD), the $attr is a BLAST
     # parameter and should be truncated to its first letter only
-
     $attr = ($attr_letter eq '_') ? $attr : $attr_letter;
     $self->throw("Unallowed parameter: $attr !") unless $OK_FIELD{$attr};
 #    $self->throw("Unallowed parameter: $attr !") unless $ok_field{$attr_letter};
     $self->{$attr_letter} = shift if @_;
-    return $self->{ $attr_letter};
+    return $self->{$attr_letter};
 }
 
 =head1 Methods
 
-=head2  exists_blast
+=head2 executable
+
+ Title   : executable
+ Usage   : my $exe = $blastfactory->executable('blastall');
+ Function: Finds the full path to the 'codeml' executable
+ Returns : string representing the full path to the exe
+ Args    : [optional] name of executable to set path to 
+           [optional] boolean flag whether or not warn when exe is not found
 
- Title   : exists_blast
- Usage   : $blastfound = Bio::Tools::Run::StandAloneBlast->exists_blast()
- Function: Determine whether Blast program can be found on current host.
-           Cf. the DESCRIPTION section of this POD for how to make sure
-           for your BLAST installation to be found. This method checks for
-           existence of the blastall executable either in BLASTDIR or in
-           the path.
-           Side effects: if BLASTDATADIR is not set, checks whether data is a
-           subdirectory of the directory where blastall is found, and if so,
-           sets DATADIR accordingly.
- Returns : 1 if Blast program found at expected location, 0 otherwise.
- Args    :  none
 
 =cut
 
-sub exists_blast {
-    my ($exe) =  shift;
-    # can call as a class method or as a function
-    if( defined $exe && $exe =~ /Bio::Tools/i ) { $exe = shift; }
-    $exe ||= 'blastall';
-    if( $^O =~ /mswin/i) { $exe .= '.exe'; }
-    my $f;
+sub executable {
+   my ($self, $exename, $exe,$warn) = @_;
+   $exename = 'blastall' unless defined $exename;
 
-    if((! $DATADIR) && (-d Bio::Root::IO->catfile($BLASTDIR, "data"))) {
-	$DATADIR = Bio::Root::IO->catfile($BLASTDIR, "data");
+   if( defined $exe && -x $exe ) {
+     $self->{'_pathtoexe'}->{$exename} = $exe;
+   }
+   unless( defined $self->{'_pathtoexe'}->{$exename} ) {
+       my $f = $self->program_path($exename);	    
+       $exe = $self->{'_pathtoexe'}->{$exename} = $f if(-e $f && -x $f );
+        
+       #  This is how I meant to split up these conditionals --jason
+       # if exe is null we will execute this (handle the case where
+       # PROGRAMDIR pointed to something invalid)
+       unless( $exe )  {  # we didn't find it in that last conditional
+	   if( ($exe = $self->io->exists_exe($exename)) && -x $exe ) {
+	       $self->{'_pathtoexe'}->{$exename} = $exe;
+	   } else { 
+	       $self->warn("Cannot find executable for $exename") if $warn;
+	       $self->{'_pathtoexe'}->{$exename} = undef;
+	   }
+       }
+   }
+   return $self->{'_pathtoexe'}->{$exename};
+}
+
+
+=head2 program_path
+
+ Title   : program_path
+ Usage   : my $path = $factory->program_path();
+ Function: Builds path for executable 
+ Returns : string representing the full path to the exe
+ Args    : none
+
+=cut
+
+sub program_path {
+    my ($self,$program_name) = @_;
+    my @path;
+    push @path, $self->program_dir if $self->program_dir;
+    push @path, $program_name .($^O =~ /mswin/i ?'.exe':'');
+
+    return Bio::Root::IO->catfile(@path);
+}
+
+=head2 program_dir
+
+ Title   : program_dir
+ Usage   : my $dir = $factory->program_dir();
+ Function: Abstract get method for dir of program. To be implemented
+           by wrapper.
+ Returns : string representing program directory 
+ Args    : none 
+
+=cut
+
+sub program_dir {
+    $PROGRAMDIR;
+}
+
+sub program {
+    my $self = shift;
+    if( wantarray ) {
+	return ($self->executable, $self->p());
+    } else {
+	return $self->executable(@_);
     }
-    
-    if( ($f = Bio::Root::IO->exists_exe($exe)) ||
-	($f = Bio::Root::IO->exists_exe(Bio::Root::IO->catfile($BLASTDIR, 
-							       $exe))) ) {
-	$PROGRAMS{$exe} = $f if( -e $f );
-	return 1;
-    }
-    #debug("exe is $exe blastall is $f\n");
-    return 0;
 }
 
 =head2  blastall
@@ -444,7 +477,7 @@ sub exists_blast {
 
 sub blastall {
     my ($self,$input1) = @_;
-    $self->_io_cleanup();
+    $self->io->_io_cleanup();
     my $executable = 'blastall';
     my $input2;
 # Create input file pointer
@@ -498,9 +531,8 @@ sub blastpgp {
 							  $mask);
     if (!$infilename1) {$self->throw(" $input1  not Bio::Seq object or array of Bio::Seq objects or file name!");}
     $self->i($infilename1);	# set file name of sequence to be blasted to inputfilename1 (-i param of blastpgp)
-
     if  ($input2) {
-	unless ($infilename2) {$self->throw("$input2  not SimpleAlign Object in pre-aligned psiblast\n");}
+	unless ($infilename2) {$self->throw("$input2 not SimpleAlign Object in pre-aligned psiblast\n");}
 	$self->B($infilename2);	# set file name of partial alignment to inputfilename2 (-B param of blastpgp)
     }
     my $blast_report = &_generic_local_blast($self, $executable, $input1, $input2);
@@ -536,11 +568,15 @@ sub bl2seq {
 # Create input file pointer
     my  ($infilename1, $infilename2 )  = $self->_setinput($executable, 
 							  $input1, $input2);
-    if (!$infilename1) {$self->throw(" $input1  not Seq Object or file name!");}
-    if  (!$infilename2) {$self->throw("$input2  not Seq Object or file name!");}
+    if (!$infilename1){$self->throw(" $input1  not Seq Object or file name!");}
+    if (!$infilename2){$self->throw("$input2  not Seq Object or file name!");}
 
-    $self->i($infilename1);	# set file name of first sequence to be aligned to inputfilename1 (-i param of bl2seq)
-    $self->j($infilename2);	# set file name of first sequence to be aligned to inputfilename2 (-j param of bl2seq)
+    $self->i($infilename1);	# set file name of first sequence to 
+                                # be aligned to inputfilename1 
+                                # (-i param of bl2seq)
+    $self->j($infilename2);	# set file name of first sequence to 
+                                # be aligned to inputfilename2 
+                                # (-j param of bl2seq)
 
     my $blast_report = &_generic_local_blast($self, $executable);    
 }
@@ -556,14 +592,13 @@ sub bl2seq {
 =cut
 
 sub _generic_local_blast {
-
     my $self = shift;
     my $executable = shift;
 
-# Create parameter string to pass to Blast program
+    # Create parameter string to pass to Blast program
     my $param_string = $self->_setparams($executable);
 
-# run Blast
+    # run Blast
     my $blast_report = &_runblast($self, $executable, $param_string);
 }
 
@@ -583,14 +618,12 @@ sub _generic_local_blast {
 
 sub _runblast {
     my ($self,$executable,$param_string) = @_;
-    my $blast_obj;
-    if( ! defined $PROGRAMS{$executable} ) {
-	unless (&Bio::Tools::Run::StandAloneBlast::exists_blast($executable)) {
-	    $self->warn("cannot find path to $executable");
-	    return undef;
-	}
+    my ($blast_obj,$exe);
+    if( ! ($exe = $self->executable($executable)) ) {
+	$self->warn("cannot find path to $executable");
+	return undef;    
     }
-    my $commandstring = $PROGRAMS{$executable} . $param_string;
+    my $commandstring = $exe. $param_string;
    
     # next line for debugging
     $self->debug( "$commandstring \n");
@@ -609,19 +642,30 @@ sub _runblast {
 # BPpsilite).  Otherwise either the Blast parser or the BPlite
 # parsers can be selected.
 
-    if ($executable eq 'bl2seq')  {
-	$blast_obj = Bio::Tools::BPbl2seq->new(-file => $outfile);
+    if ($executable =~ /bl2seq/i)  {
+        if( $self->verbose > 0 ) {
+	 open(OUT, $outfile) || $self->throw("cannot open $outfile");
+	 while(<OUT>) { $self->debug($_)}
+	 close(OUT);
+        }
+# Added program info so BPbl2seq can compute strand info
+	$blast_obj = Bio::Tools::BPbl2seq->new(-file => $outfile,
+                                               -REPORT_TYPE => $self->p );
+#	$blast_obj = Bio::Tools::BPbl2seq->new(-file => $outfile);
     }
-    elsif ($executable eq 'blastpgp' && defined $self->j() && 
+    elsif ($executable =~ /blastpgp/i && defined $self->j() && 
 	   $self->j() > 1)  {
+	print "using psilite parser\n";
 	$blast_obj = Bio::Tools::BPpsilite->new(-file => $outfile);
     }
-    elsif ($self->_READMETHOD eq 'Blast')  {
+    elsif ($self->_READMETHOD =~ /^Blast/i )  {
 	$blast_obj = Bio::SearchIO->new(-file=>$outfile,
 					-format => 'blast'   )  ;
     }
-    elsif ($self->_READMETHOD eq 'BPlite')  {
+    elsif ($self->_READMETHOD =~ /^BPlite/i )  {
 	$blast_obj = Bio::Tools::BPlite->new(-file=>$outfile);
+    } else {
+	$self->warn("Unrecognized readmethod ".$self->_READMETHOD. " or executable $executable\n");
     }
     return $blast_obj;
 }
@@ -642,7 +686,7 @@ sub _setinput {
     my ($seq, $temp, $infilename1, $infilename2,$fh ) ;
 #  If $input1 is not a reference it better be the name of a file with
 #  the sequence/ alignment data...
-    $self->_io_cleanup();
+    $self->io->_io_cleanup();
 
   SWITCH:  {
       unless (ref $input1) {
@@ -651,31 +695,31 @@ sub _setinput {
       }
 #  $input may be an array of BioSeq objects...
       if (ref($input1) =~ /ARRAY/i ) {
-	  ($fh,$infilename1) = $self->tempfile();
+	  ($fh,$infilename1) = $self->io->tempfile();
 	  $temp =  Bio::SeqIO->new(-fh=> $fh, '-format' => 'Fasta');
 	  foreach $seq (@$input1) {
 	      unless ($seq->isa("Bio::PrimarySeqI")) {return 0;}
 	      $temp->write_seq($seq);
 	  }
 	  close $fh;
+	  $fh = undef;
 	  last SWITCH;
       }
 #  $input may be a single BioSeq object...
       elsif ($input1->isa("Bio::PrimarySeqI")) {
-	  ($fh,$infilename1) = $self->tempfile();
+	  ($fh,$infilename1) = $self->io->tempfile();
 
 # just in case $input1 is taken from an alignment and has spaces (ie
 # deletions) indicated within it, we have to remove them - otherwise
 # the BLAST programs will be unhappy
 
-	  my $seq_id =  $input1->display_id();
 	  my $seq_string =  $input1->seq();
 	  $seq_string =~ s/\W+//g; # get rid of spaces in sequence
-	  $seq = Bio::Seq->new(-seq=> $seq_string, -display_id =>$seq_id );
-
+	  $input1->seq($seq_string);
 	  $temp =  Bio::SeqIO->new(-fh=> $fh, '-format' => 'Fasta');
-	  $temp->write_seq($seq);
+	  $temp->write_seq($input1);
 	  close $fh;
+	  undef $fh;
 #		$temp->write_seq($input1);
 	  last SWITCH;
       }
@@ -688,30 +732,31 @@ sub _setinput {
 	  last SWITCH2; 
       }
       if ($input2->isa("Bio::PrimarySeqI")  && $executable  eq 'bl2seq' ) {
-	  ($fh,$infilename2) = $self->tempfile();
+	  ($fh,$infilename2) = $self->io->tempfile();
 
 	  $temp =  Bio::SeqIO->new(-fh=> $fh, '-format' => 'Fasta');
 	  $temp->write_seq($input2);
 	  close $fh;
+	  undef $fh;
 	  last SWITCH2;
       }
 # Option for using psiblast's pre-alignment "jumpstart" feature
       elsif ($input2->isa("Bio::SimpleAlign")  && 
 	     $executable  eq 'blastpgp' ) {
-# a bit of a lie since it won't be a fasta file
-	  ($fh,$infilename2) = $self->tempfile(); 
+           # a bit of a lie since it won't be a fasta file
+	  ($fh,$infilename2) = $self->io->tempfile(); 
 
 # first we retrieve the "mask" that determines which residues should
 # by scored according to their position and which should be scored
 # using the non-position-specific matrices
 
-	  my @mask = split("", shift ); #  get mask
+	  my @mask = split("", shift );	#  get mask
 
 # then we have to convert all the residues in every sequence to upper
 # case at the positions that we want psiblast to use position specific
 # scoring
 
-	  foreach $seq ( $input2->eachSeq() ) {
+	  foreach $seq ( $input2->each_seq() ) {
 	      my @seqstringlist = split("",$seq->seq());
 	      for (my $i = 0; $i < scalar(@mask); $i++) {
 		  unless ( $seqstringlist[$i] =~ /[a-zA-Z]/ ) {next}
@@ -720,10 +765,13 @@ sub _setinput {
 	      my $newseqstring = join("", @seqstringlist);
 	      $seq->seq($newseqstring);
 	  }
-#  Now we need to write out the alignment to a file in the "psi format" which psiblast is expecting
+          #  Now we need to write out the alignment to a file 
+          # in the "psi format" which psiblast is expecting
+	  $input2->map_chars('\.','-');
 	  $temp =  Bio::AlignIO->new(-fh=> $fh, '-format' => 'psi');
 	  $temp->write_aln($input2);
 	  close $fh;
+	  undef $fh;
 	  last SWITCH2;
       }
       $infilename2 = 0;		# Set error flag if you get here
@@ -771,5 +819,87 @@ sub _setparams {
     return $param_string;
 }
 
+
+=head1 Bio::Tools::Run::Wrapper methods
+
+=cut
+
+=head2 no_param_checks
+
+ Title   : no_param_checks
+ Usage   : $obj->no_param_checks($newval)
+ Function: Boolean flag as to whether or not we should
+           trust the sanity checks for parameter values  
+ Returns : value of no_param_checks
+ Args    : newvalue (optional)
+
+
+=cut
+
+=head2 save_tempfiles
+
+ Title   : save_tempfiles
+ Usage   : $obj->save_tempfiles($newval)
+ Function: 
+ Returns : value of save_tempfiles
+ Args    : newvalue (optional)
+
+
+=cut
+
+=head2 outfile_name
+
+ Title   : outfile_name
+ Usage   : my $outfile = $tcoffee->outfile_name();
+ Function: Get/Set the name of the output file for this run
+           (if you wanted to do something special)
+ Returns : string
+ Args    : [optional] string to set value to
+
+
+=cut
+
+
+=head2 tempdir
+
+ Title   : tempdir
+ Usage   : my $tmpdir = $self->tempdir();
+ Function: Retrieve a temporary directory name (which is created)
+ Returns : string which is the name of the temporary directory
+ Args    : none
+
+
+=cut
+
+=head2 cleanup
+
+ Title   : cleanup
+ Usage   : $tcoffee->cleanup();
+ Function: Will cleanup the tempdir directory after a PAML run
+ Returns : none
+ Args    : none
+
+
+=cut
+
+=head2 io
+
+ Title   : io
+ Usage   : $obj->io($newval)
+ Function:  Gets a L<Bio::Root::IO> object
+ Returns : L<Bio::Root::IO>
+ Args    : none
+
+
+=cut
+
+sub DESTROY {
+    my $self= shift;
+    unless ( $self->save_tempfiles ) {
+	$self->cleanup();
+    }
+    $self->SUPER::DESTROY();
+}
+
 1;
 __END__
