@@ -1,4 +1,4 @@
-# $Id: xml.pm,v 1.11 2001/10/22 08:22:57 heikki Exp $
+# $Id$
 # BioPerl module for Bio::Variation::IO::xml
 #
 # Cared for by Heikki Lehvaslaiho <Heikki@ebi.ac.uk>
@@ -50,7 +50,7 @@ report bugs to the Bioperl bug tracking system to help us keep track
  email or the web:
 
   bioperl-bugs@bio.perl.org
-  http://bio.perl.org/bioperl-bugs/
+  http://bugzilla.bioperl.org/
 
 =head1 AUTHOR - Heikki Lehvaslaiho
 
@@ -243,13 +243,12 @@ sub next {
 sub write {
     my ($self,@h) = @_;
 
-    if( !defined $h[0] ) {
+    if( ! defined $h[0] ) {
         $self->throw("Attempting to write with no information!");
     }
     my $str;
     my $output = IO::String->new($str);
     my $w = new XML::Writer(OUTPUT => $output, DATA_MODE => 1, DATA_INDENT => 4 );
-
     foreach my $h (@h) {
 	#
 	# seqDiff
@@ -553,10 +552,9 @@ sub write {
     }
     $w->endTag;
 
-
     $w->end;
     $self->_print($str);
-    $output->close;
+    $output = undef;
     return 1;
 }
 
